"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

class _ProtocolConstants:
    ValueType = _typing.NewType("ValueType", _builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _ProtocolConstantsEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[_ProtocolConstants.ValueType], _builtins.type):
    DESCRIPTOR: _descriptor.EnumDescriptor
    UNKNOWN: _ProtocolConstants.ValueType  # 0
    SERVER_VERSION: _ProtocolConstants.ValueType  # 143165457
    MESSAGE_MAGIC: _ProtocolConstants.ValueType  # 19088745
    REP_ERROR: _ProtocolConstants.ValueType  # 4096
    RPC_PTY_BUFFER_SIZE: _ProtocolConstants.ValueType  # 65536
    RPC_MAX_REQ_MSG_ID: _ProtocolConstants.ValueType  # 256

class ProtocolConstants(_ProtocolConstants, metaclass=_ProtocolConstantsEnumTypeWrapper): ...

UNKNOWN: ProtocolConstants.ValueType  # 0
SERVER_VERSION: ProtocolConstants.ValueType  # 143165457
MESSAGE_MAGIC: ProtocolConstants.ValueType  # 19088745
REP_ERROR: ProtocolConstants.ValueType  # 4096
RPC_PTY_BUFFER_SIZE: ProtocolConstants.ValueType  # 65536
RPC_MAX_REQ_MSG_ID: ProtocolConstants.ValueType  # 256
Global___ProtocolConstants: _TypeAlias = ProtocolConstants  # noqa: Y015

class _Arch:
    ValueType = _typing.NewType("ValueType", _builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _ArchEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[_Arch.ValueType], _builtins.type):
    DESCRIPTOR: _descriptor.EnumDescriptor
    ARCH_UNKNOWN: _Arch.ValueType  # 0
    ARCH_ARM64: _Arch.ValueType  # 1

class Arch(_Arch, metaclass=_ArchEnumTypeWrapper): ...

ARCH_UNKNOWN: Arch.ValueType  # 0
ARCH_ARM64: Arch.ValueType  # 1
Global___Arch: _TypeAlias = Arch  # noqa: Y015

@_typing.final
class RpcMessage(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    MAGIC_FIELD_NUMBER: _builtins.int
    CLIENT_ID_FIELD_NUMBER: _builtins.int
    MSG_ID_FIELD_NUMBER: _builtins.int
    PAYLOAD_FIELD_NUMBER: _builtins.int
    magic: _builtins.int
    client_id: _builtins.int
    msg_id: _builtins.int
    payload: _builtins.bytes
    def __init__(
        self,
        *,
        magic: _builtins.int = ...,
        client_id: _builtins.int = ...,
        msg_id: _builtins.int = ...,
        payload: _builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["client_id", b"client_id", "magic", b"magic", "msg_id", b"msg_id", "payload", b"payload"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RpcMessage: _TypeAlias = RpcMessage  # noqa: Y015

@_typing.final
class RpcPtyMessage(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    BUFFER_FIELD_NUMBER: _builtins.int
    EXIT_CODE_FIELD_NUMBER: _builtins.int
    buffer: _builtins.bytes
    exit_code: _builtins.int
    def __init__(
        self,
        *,
        buffer: _builtins.bytes = ...,
        exit_code: _builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["buffer", b"buffer", "exit_code", b"exit_code", "type", b"type"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["buffer", b"buffer", "exit_code", b"exit_code", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["buffer", "exit_code"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___RpcPtyMessage: _TypeAlias = RpcPtyMessage  # noqa: Y015

@_typing.final
class Handshake(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    MAGIC_FIELD_NUMBER: _builtins.int
    ARCH_FIELD_NUMBER: _builtins.int
    SYSNAME_FIELD_NUMBER: _builtins.int
    MACHINE_FIELD_NUMBER: _builtins.int
    PLATFORM_FIELD_NUMBER: _builtins.int
    SERVER_VERSION_FIELD_NUMBER: _builtins.int
    CLIENT_ID_FIELD_NUMBER: _builtins.int
    magic: _builtins.int
    arch: Global___Arch.ValueType
    sysname: _builtins.str
    machine: _builtins.str
    platform: _builtins.str
    server_version: _builtins.int
    client_id: _builtins.int
    def __init__(
        self,
        *,
        magic: _builtins.int = ...,
        arch: Global___Arch.ValueType = ...,
        sysname: _builtins.str = ...,
        machine: _builtins.str = ...,
        platform: _builtins.str = ...,
        server_version: _builtins.int = ...,
        client_id: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["arch", b"arch", "client_id", b"client_id", "machine", b"machine", "magic", b"magic", "platform", b"platform", "server_version", b"server_version", "sysname", b"sysname"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___Handshake: _TypeAlias = Handshake  # noqa: Y015



import time
from typing import TYPE_CHECKING, Union

from rpcclient.clients.darwin.consts import kCGHIDEventTap, kCGNullWindowID, kCGWindowListOptionAll
from rpcclient.exceptions import BadReturnValueError

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class CoreGraphics:
    """Manage Core Graphics events."""

    def __init__(self, client: "DarwinClient"):
        self._client = client

    @property
    def window_list(self):
        """
        get a list of all opened windows
        https://developer.apple.com/documentation/coregraphics/1455137-cgwindowlistcopywindowinfo?language=objc
        """
        return self._client.symbols.CGWindowListCopyWindowInfo(kCGWindowListOptionAll, kCGNullWindowID).py()

    def send_key_press(self, key_code: int, interval: Union[float, int] = 0):
        """
        Send a key-press event.
        Accessibility features must be allowed.
        """
        self.send_keyboard_event(key_code, True)
        if interval:
            time.sleep(interval)
        self.send_keyboard_event(key_code, False)

    def send_keyboard_event(self, key_code: int, down: bool):
        """
        send a CG keyboard event
        https://developer.apple.com/documentation/coregraphics/1456564-cgeventcreatekeyboardevent
        """
        event = self._client.symbols.CGEventCreateKeyboardEvent(0, key_code, down)
        if not event:
            raise BadReturnValueError("CGEventCreateKeyboardEvent() failed")

        self._client.symbols.CGEventPost(kCGHIDEventTap, event)



import plistlib
from contextlib import suppress

import pytest

from rpcclient.exceptions import NoSuchPreferenceError

pytestmark = pytest.mark.darwin
DOMAIN = "rpcserver"
USERNAME = "kCFPreferencesAnyUser"
PLIST_PATH = f"/Library/Preferences/{DOMAIN}.plist"


@pytest.fixture()
def tmp_preference(client):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    client.preferences.cf.set_dict({"key1": "value1"}, DOMAIN, USERNAME)
    client.preferences.cf.sync(DOMAIN, USERNAME)
    client.fs.write_file(PLIST_PATH, plistlib.dumps({"key1": "value1"}))
    try:
        yield
    finally:
        with suppress(NoSuchPreferenceError):
            client.preferences.cf.clear(DOMAIN, USERNAME)
        client.preferences.cf.sync(DOMAIN, USERNAME)


def test_cf_get_keys(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    assert "key1" in client.preferences.cf.get_keys(DOMAIN, USERNAME)


def test_cf_get_keys_invalid_preference(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with pytest.raises(NoSuchPreferenceError):
        client.preferences.cf.get_keys("com.apple.invalid_preference_for_sure", USERNAME)


def test_cf_get_value(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    assert client.preferences.cf.get_value("key1", DOMAIN, USERNAME) == "value1"


def test_cf_get_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    assert client.preferences.cf.get_dict(DOMAIN, USERNAME)["key1"] == "value1"


def test_cf_set(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    client.preferences.cf.set("key2", {"hey": "you"}, DOMAIN, USERNAME)
    assert client.preferences.cf.get_value("key2", DOMAIN, USERNAME) == {"hey": "you"}


def test_cf_remove(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    client.preferences.cf.remove("key1", DOMAIN, USERNAME)
    assert client.preferences.cf.get_value("key1", DOMAIN, USERNAME) is None


def test_cf_set_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    client.preferences.cf.get_value("key1", DOMAIN, USERNAME)
    client.preferences.cf.set_dict({"b": 5}, DOMAIN, USERNAME)
    assert client.preferences.cf.get_dict(DOMAIN, USERNAME) == {"b": 5}


def test_cf_update_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    update_contents = {"a": 5}
    expected_dict = client.preferences.cf.get_dict(DOMAIN, USERNAME)
    expected_dict.update(update_contents)
    client.preferences.cf.update_dict(update_contents, DOMAIN, USERNAME)
    assert client.preferences.cf.get_dict(DOMAIN, USERNAME) == expected_dict


def test_cf_clear(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    client.preferences.cf.clear(DOMAIN, USERNAME)
    with pytest.raises(NoSuchPreferenceError):
        assert not client.preferences.cf.get_dict(DOMAIN, USERNAME)


def test_sc_get_keys(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    keys = client.preferences.sc.get_keys(PLIST_PATH)
    assert "key1" in keys


def test_sc_get_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    dict_ = client.preferences.sc.get_dict(PLIST_PATH)
    assert "key1" in dict_


def test_sc_object_keys(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        keys = pref.keys
    assert "key1" in keys


def test_sc_object_set(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        pref.set("key2", "value2")
    assert client.preferences.sc.get_dict(PLIST_PATH)["key2"] == "value2"


def test_sc_object_set_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        pref.set_dict({"hey": "you"})
    assert client.preferences.sc.get_dict(PLIST_PATH) == {"hey": "you"}


def test_sc_object_update_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        pref.update_dict({"hey": "you"})
    dict_ = client.preferences.sc.get_dict(PLIST_PATH)
    assert dict_["key1"] == "value1"
    assert dict_["hey"] == "you"


def test_sc_object_remove(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        pref.remove("key1")
    assert "key1" not in client.preferences.sc.get_dict(PLIST_PATH)


def test_sc_object_get(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        val = pref.get("key1")
    assert val == "value1"


def test_sc_object_get_dict(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        dict_ = pref.get_dict()
    assert dict_ == {"key1": "value1"}


def test_sc_object_clear(client, tmp_preference):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    with client.preferences.sc.open(PLIST_PATH) as pref:
        pref.clear()
    assert not client.preferences.sc.get_dict(PLIST_PATH)


class TestCustomDomain:
    @pytest.fixture(autouse=True)
    def clear_domain(self, client):
        with suppress(NoSuchPreferenceError):
            # if from some reason this domain already exist, empty it
            client.preferences.cf.clear(DOMAIN)
            assert client.preferences.cf.get_keys(DOMAIN) is None

    def test_set_dict(self, client):
        test_dict = {"KEY1": "VALUE1", "KEY2": "VALUE2"}
        client.preferences.cf.set_dict(test_dict, DOMAIN)
        assert test_dict == client.preferences.cf.get_dict(DOMAIN)

    def test_remove(self, client):
        test_dict = {"KEY1": "VALUE1", "KEY2": "VALUE2"}
        client.preferences.cf.set_dict(test_dict, DOMAIN)
        client.preferences.cf.remove("KEY2", DOMAIN)
        test_dict.pop("KEY2")
        assert test_dict == client.preferences.cf.get_dict(DOMAIN)


def test_sc_preferences(client):
    with client.preferences.sc.open(DOMAIN) as pref:
        if pref.get_dict() != {}:
            # if from some reason this domain already exist, empty it
            pref.clear()
            assert pref.get_dict() == {}

        # test set a full dictionary
        test_dict = {"KEY1": "VALUE1", "KEY2": "VALUE2"}
        pref.set_dict(test_dict)
        assert test_dict == pref.get_dict()

        # test set remove a single value
        pref.remove("KEY2")
        test_dict.pop("KEY2")
        assert test_dict == pref.get_dict()

        # remove out test data and verify it works
        pref.clear()
        assert pref.get_dict() == {}



from collections import namedtuple
from typing import TYPE_CHECKING

from rpcclient.exceptions import RpcFailedLaunchingAppError

if TYPE_CHECKING:
    from rpcclient.clients.ios.client import IosClient

ScreenLockStatus = namedtuple("ScreenLockStatus", ["lock", "passcode"])


class SpringBoard:
    """SpringBoardServices helpers."""

    def __init__(self, client: "IosClient") -> None:
        self._client = client

    def get_spring_board_server_port(self) -> int:
        """Return the SpringBoard server port."""
        return self._client.symbols.SBSSpringBoardServerPort()

    def launch_application(self, bundle_identifier: str) -> None:
        """Launch an app via SpringBoardServices."""
        err = self._client.symbols.SBSLaunchApplicationWithIdentifier(self._client.cf(bundle_identifier), 0)
        if err != 0:
            raise RpcFailedLaunchingAppError(
                f"SBSLaunchApplicationWithIdentifier failed with: error code {err} - "
                f"{self._client.symbols.SBSApplicationLaunchingErrorString(err).py()}"
            )

    def get_screen_lock_status(self) -> ScreenLockStatus:
        """Return lockscreen and passcode status via SpringBoardServices."""
        server_port = self.get_spring_board_server_port()
        with self._client.safe_malloc(8) as p_is_lock, self._client.safe_malloc(8) as p_is_passcode:
            p_is_lock[0] = 0
            p_is_passcode[0] = 0
            self._client.symbols.SBGetScreenLockStatus(server_port, p_is_lock, p_is_passcode)
            return ScreenLockStatus(p_is_lock[0] == 1, p_is_passcode[0] == 1)

    def open_sensitive_url_and_unlock(self, url: str, unlock: bool = True) -> None:
        """Open a URL with the system handler, optionally unlocking."""
        screen_lock_status = self.get_screen_lock_status()
        if not unlock and screen_lock_status.lock:
            if screen_lock_status.passcode:
                raise RpcFailedLaunchingAppError(
                    "cannot open url while screen is locked with passcode. you must unlock device first"
                )
            raise RpcFailedLaunchingAppError("cannot open url while screen is locked, use unlock=True parameter")
        cf_url_ref = self._client.symbols.objc_getClass("NSURL").objc_call("URLWithString:", self._client.cf(url))
        if not self._client.symbols.SBSOpenSensitiveURLAndUnlock(cf_url_ref, unlock):
            raise RpcFailedLaunchingAppError("SBSOpenSensitiveURLAndUnlock failed")



from dataclasses import dataclass
from typing import Any, Callable

from prompt_toolkit.enums import DEFAULT_BUFFER
from prompt_toolkit.filters import EmacsInsertMode, HasFocus, HasSelection, ViInsertMode
from prompt_toolkit.keys import Keys


@dataclass
class Keybinding:
    """Single keybinding entry for the console."""

    key: str
    description: str
    callback: Callable[..., None]


def get_keybindings(console: Any) -> list[Keybinding]:
    """Return the list of default keybindings for the given console."""
    return [
        Keybinding(key=Keys.F1, description="Show this help", callback=console.show_help),
        Keybinding(key=Keys.F2, description="Show active contexts", callback=console.show_contexts),
        Keybinding(key=Keys.F3, description="Previous context", callback=console.previous_context),
        Keybinding(key=Keys.F4, description="Toggle Auto switch on creation", callback=console.toggle_auto_switch),
    ]


def load_ipython_extension(ipython: Any) -> None:
    """IPython entry-point to register console keybindings."""

    def register_keybindings() -> None:
        """Register keybindings on the running IPython application."""
        console = ipython.user_ns["console"]
        insert_mode = ViInsertMode() | EmacsInsertMode()
        registry = ipython.pt_app.key_bindings

        for keybind in get_keybindings(console):
            registry.add_binding(
                keybind.key,
                filter=(HasFocus(DEFAULT_BUFFER) & ~HasSelection() & insert_mode),
            )(keybind.callback)

    register_keybindings()
    ipython.events.register("shell_initialized", register_keybindings)



import logging
import threading
from typing import Callable, Union

from rpcclient.clients.ios.client import IosClient
from rpcclient.clients.linux.client import LinuxClient
from rpcclient.clients.macos.client import MacosClient
from rpcclient.core.client import ClientEvent, CoreClient
from rpcclient.event_notifier import EventNotifier
from rpcclient.protocol.rpc_bridge import RpcBridge
from rpcclient.registry import Registry
from rpcclient.transports import create_local, create_tcp, create_using_protocol
from rpcclient.utils import prompt_selection

logger = logging.getLogger(__name__)
ClientType = Union[IosClient, MacosClient, LinuxClient, CoreClient]
ClientClass = type[ClientType]


class ClientManager:
    """Manage client lifecycle and dispatch client-related events."""

    def __init__(self) -> None:
        """Initialize registries, notifier, and register default factories/transports."""
        self.notifier = EventNotifier()
        self.client_factory = Registry[str, ClientClass]({
            "ios": IosClient,
            "osx": MacosClient,
            "linux": LinuxClient,
            "core": CoreClient,
        })

        self.transport_factory = Registry[str, Callable[..., RpcBridge]]({
            "tcp": create_tcp,
            "local": create_local,
            "protocol": create_using_protocol,
        })

        self._lock = threading.RLock()
        self._clients: Registry[int, ClientType] = Registry(notifier=self.notifier)

    def create(self, mode: str = "tcp", internal: bool = False, **kwargs) -> ClientType:
        """
        Create a client via transport `mode`, resolve platform, store, and emit CREATED.

        :param mode: Transport mode, e.g. "tcp" or "protocol" (default: "tcp")
        :param internal: If True, supply the CREATED event with the internal flag (default: False)
        """
        transport_factory = self.transport_factory.get(mode)
        if transport_factory is None:
            raise ValueError(f"Unknown client mode: {mode}")

        # If using protocol-based spawn/routing and no explicit client provided,
        # try to pick a capable existing client (with create_worker).
        if mode == "protocol" and "client" not in kwargs:
            kwargs["client"] = self._select_capable_client()

        rpc_bridge = transport_factory(**kwargs)
        server_type = rpc_bridge.platform
        cached = self.get(rpc_bridge.client_id)
        if cached is not None:
            return cached

        client_factory = self.client_factory.get(server_type)
        if client_factory is None:
            raise ValueError(f"Unknown client mode: {server_type}")

        client: ClientType = client_factory(bridge=rpc_bridge)
        client.notifier.register(ClientEvent.TERMINATED, self._on_client_terminated)
        client.notifier.register(ClientEvent.CREATED, self._on_client_created)
        self.add(client, internal=internal)

        return client

    def _select_capable_client(self):
        capable = [c for c in self.clients.values() if hasattr(c, "create_worker") and callable(c.create_worker)]
        if not capable:
            raise ValueError("No existing client supports protocol worker creation.")
        if len(capable) == 1:
            return capable[0]
        else:
            return prompt_selection(capable, "Select a client client ID")

    def add(self, client: ClientType, internal: bool = False) -> None:
        """
        Add a client to the registry; emit REGISTERED if successful.

        :param internal: If True, supply the REGISTERED event with the internal flag (default: False)
        """
        self._clients.register(client.id, client, internal=internal)

    def remove(self, cid: int) -> None:
        """Remove a client by ID; emit REMOVED if found."""
        self._clients.unregister(cid)

    def clear(self) -> None:
        """Remove all clients."""
        self._clients.clear()

    def get(self, cid: int) -> Union[ClientType, None]:
        """Return the client for ID, or None."""
        return self._clients.get(cid)

    @property
    def clients(self) -> dict[int, ClientType]:
        return dict(self._clients.items())

    # ---------------------------------------------------------------------------
    # Client Event callbacks
    # ---------------------------------------------------------------------------

    def _on_client_terminated(self, cid: int) -> None:
        """Internal: remove a client when it terminates."""
        self.remove(cid)

    def _on_client_created(self, client: ClientType) -> None:
        """Internal: Add a client when if created by another client."""
        self.add(client)



from io import StringIO

import pytest


def test_spawn_fds(client):
    pid = client.spawn(["/bin/sleep", "5"], stdout=StringIO(), stdin="", background=True).pid

    # should only have: stdin, stdout and stderr
    assert len(client.processes.get_by_pid(pid).fds) == 3

    client.processes.kill(pid)


@pytest.mark.local_only
@pytest.mark.parametrize(
    "argv,expected_stdout,success",
    [
        [["/bin/sleep", "0"], "", True],
        [["/bin/echo", "blat"], "blat", True],
        [["/bin/ls", "INVALID_PATH"], "No such file or directory", False],
    ],
)
def test_spawn_foreground_sanity(client, argv, expected_stdout, success):
    stdout = StringIO()
    if success:
        assert client.spawn(argv, stdout=stdout, stdin="").error == 0
    else:
        assert client.spawn(argv, stdout=stdout, stdin="").error != 0
    stdout.seek(0)
    assert expected_stdout in stdout.read().strip()


@pytest.mark.local_only
@pytest.mark.parametrize(
    "argv,expected_stdout,success",
    [
        [["/bin/sleep", "0"], "", True],
        [["/bin/echo", "blat"], "blat", True],
        [["/bin/ls", "INVALID_PATH"], "No such file or directory", False],
    ],
)
def test_spawn_foreground_stress(client, argv, expected_stdout, success):
    for _i in range(100):
        test_spawn_foreground_sanity(client, argv, expected_stdout, success)


def test_spawn_background_sanity(client):
    spawn_result = client.spawn(["/bin/sleep", "5"], stdout=StringIO(), stdin="", background=True)

    # when running in background, no error is returned
    assert spawn_result.error is None
    assert spawn_result.stdout is None

    # instead, we can just make sure it ran by sending it a kill and don't fail
    client.processes.kill(spawn_result.pid)


def test_spawn_background_stress(client):
    for _i in range(100):
        test_spawn_background_sanity(client)



from collections import namedtuple
from dataclasses import dataclass, field

from objc_types_decoder.decode import decode as decode_type
from objc_types_decoder.decode import decode_with_tail

from rpcclient.core.client import CoreClient

Property = namedtuple("Property", "name attributes")
PropertyAttributes = namedtuple("PropertyAttributes", "synthesize type_ list")


def convert_encoded_property_attributes(encoded):
    conversions = {
        "R": lambda x: "readonly",
        "C": lambda x: "copy",
        "&": lambda x: "strong",
        "N": lambda x: "nonatomic",
        "G": lambda x: "getter=" + x[1:],
        "S": lambda x: "setter=" + x[1:],
        "d": lambda x: "dynamic",
        "W": lambda x: "weak",
        "P": lambda x: "<garbage-collected>",
        "t": lambda x: "encoding=" + x[1:],
    }

    type_, tail = decode_with_tail(encoded[1:])
    attributes = []
    synthesize = None
    for attr in filter(None, tail.lstrip(",").split(",")):
        if attr[0] in conversions:
            attributes.append(conversions[attr[0]](attr))
        elif attr[0] == "V":
            synthesize = attr[1:]

    return PropertyAttributes(type_=type_, synthesize=synthesize, list=attributes)


@dataclass
class Method:
    name: str
    client: CoreClient = field(compare=False)
    address: int = field(compare=False)
    imp: int = field(compare=False)
    type_: str = field(compare=False)
    return_type: str = field(compare=False)
    is_class: bool = field(compare=False)
    args_types: list = field(compare=False)

    @staticmethod
    def from_data(data: dict, client):
        """
        Create Method object from raw data.
        :param data: Data as loaded from get_objectivec_symbol_data.m.
        :param rpcclient.darwin.client.DarwinClient client: Darwin client.
        """
        return Method(
            name=data["name"],
            client=client,
            address=client.symbol(data["address"]),
            imp=client.symbol(data["imp"]),
            type_=data["type"],
            return_type=decode_type(data["return_type"]),
            is_class=data["is_class"],
            args_types=list(map(decode_type, data["args_types"])),
        )

    def set_implementation(self, new_imp: int):
        self.client.symbols.method_setImplementation(self.address, new_imp)
        self.imp = self.client.symbol(new_imp)

    def always_return(self, value: bool) -> None:
        """
        Patch the method to always return the given value.
        """
        # Exported from rpcserver binary
        ret_val = self.client.symbols.get_true if value else self.client.symbols.get_false
        self.set_implementation(ret_val)

    def __str__(self):
        if ":" in self.name:
            args_names = self.name.split(":")
            name = " ".join(["{}:({})".format(*arg) for arg in zip(args_names, self.args_types[2:])])
        else:
            name = self.name
        prefix = "+" if self.is_class else "-"
        return f"{prefix} {name}; // 0x{self.address:x} (returns: {self.return_type})\n"



from typing import TYPE_CHECKING

from construct import (
    Bytes,
    CString,
    Default,
    FlagsEnum,
    FormatField,
    If,
    Int8ul,
    Int16sl,
    Int16ub,
    Int16ul,
    Int32ub,
    Int32ul,
    Int64sl,
    Int64ul,
    LazyBound,
    PaddedString,
    Padding,
    Pointer,
    Struct,
    this,
)

from rpcclient.core.structs.consts import AF_INET, AF_INET6, AF_UNIX

if TYPE_CHECKING:
    from rpcclient.core.client import CoreClient

UNIX_PATH_MAX = 104

u_char = Int8ul
uint8_t = Int8ul
short = Int16sl
u_short = Int16ul
uint32_t = Int32ul
uint64_t = Int64ul
u_int32_t = uint32_t
uid_t = Int32ul
gid_t = Int32ul
time_t = Int32ul
long = Int64sl
mode_t = Int16ul
in_addr = Bytes(4)
in6_addr = Bytes(16)

sockaddr_in = Struct(
    "sin_family" / Default(Int16sl, AF_INET),
    "sin_port" / Int16ub,
    "sin_addr" / in_addr,
    "sin_zero" / Default(Bytes(8), b"\x00" * 8),
)

sockaddr_in6 = Struct(
    "sin6_family" / Default(Int16sl, AF_INET6),
    "sin6_port" / Int16ub,
    "sin6_flowinfo" / Default(Int32ub, 0),
    "sin6_addr" / in6_addr,
    "sin6_scope_id" / Default(Int32ub, 0),
)

sockaddr_un = Struct(
    "sun_len" / Default(Int8ul, 0),
    "sun_family" / Default(Int8ul, AF_UNIX),
    "sun_path" / PaddedString(UNIX_PATH_MAX, "utf8"),
)

sockaddr = Struct(
    Padding(1),
    "sa_family" / Int8ul,
)

st_flags = FlagsEnum(
    Int32ul,
    UF_NODUMP=1,
    UF_IMMUTABLE=2,
    UF_APPEND=4,
    SF_ARCHIVED=0x00010000,
    SF_IMMUTABLE=0x00020000,
    SF_APPEND=0x00040000,
)

block_descriptor = Struct("reserved" / Int64ul, "size" / Int64ul)
block_literal = Struct(
    "isa" / Int64ul,
    "flags" / Int32ul,
    "reserved" / Int32ul,
    "invoke" / Int64ul,
    "descriptor" / Int64ul,
)


class SymbolFormatField(FormatField):
    """
    A Symbol wrapper for construct
    """

    def __init__(self, client: "CoreClient"):
        super().__init__("<", "Q")
        self._client = client

    def _parse(self, stream, context, path):
        return self._client.symbol(FormatField._parse(self, stream, context, path))


def hostent(client) -> Struct:
    return Struct(
        "_h_name" / SymbolFormatField(client),
        "h_name" / Pointer(this._h_name, CString("utf8")),
        "h_aliases" / SymbolFormatField(client),
        "h_addrtype" / Int32ul,
        "h_length" / Int32ul,
        "h_addr_list" / SymbolFormatField(client),
    )


def ifaddrs(client) -> Struct:
    return Struct(
        "_ifa_next" / SymbolFormatField(client),
        "ifa_next" / If(this._ifa_next != 0, LazyBound(lambda: Pointer(this._ifa_next, ifaddrs(client)))),
        "_ifa_name" / SymbolFormatField(client),
        "ifa_name" / If(this._ifa_name != 0, Pointer(this._ifa_name, CString("utf8"))),
        "ifa_flags" / Int32ul,
        Padding(4),
        "ifa_addr" / SymbolFormatField(client),
        "ifa_netmask" / SymbolFormatField(client),
        "ifa_dstaddr" / SymbolFormatField(client),
        "ifa_data" / SymbolFormatField(client),
    )


def Dl_info(client) -> Struct:
    return Struct(
        "_dli_fname" / SymbolFormatField(client),
        "dli_fname" / Pointer(this._dli_fname, CString("utf8")),
        "dli_fbase" / SymbolFormatField(client),
        "_dli_sname" / SymbolFormatField(client),
        "dli_sname" / Pointer(this._dli_sname, CString("utf8")),
        "dli_saddr" / SymbolFormatField(client),
    )



import json
from datetime import datetime
from functools import cached_property
from typing import TYPE_CHECKING, Any, Optional

from rpcclient.clients.darwin.symbol import DarwinSymbol
from rpcclient.core.allocated import Allocated
from rpcclient.core.subsystems.fs import Fs, RemotePath
from rpcclient.exceptions import SymbolAbsentError

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient

BIOME_PATHS = [
    "/private/var/mobile/Library/Biome/streams/restricted",
    "/private/var/db/biome/streams/restricted",
    "/private/var/mobile/Library/Biome/streams/public",
    "/private/var/db/biome/streams/public",
]


class Biome:
    """
    Biome interface for interacting with BiomeStreams.

    Loads the required framework and exposes helpers for enumerating streams and
    creating a `BiomeLibrary` for reading events.

    :param rpcclient.darwin.client.DarwinClient client: Connected Darwin client.
    :param Optional[list[str]] biome_paths: Paths to Biome storage directories on the remote host.
    :param Optional[Fs] fs: Filesystem helper. Defaults to `client.fs`.
    """

    def __init__(
        self, client: "DarwinClient", biome_paths: Optional[list[str]] = None, fs: Optional[Fs] = None
    ) -> None:
        self._client = client
        self._client.load_framework("BiomeStreams")
        self.fs = fs if fs is not None else client.fs
        self.biome_paths = biome_paths if biome_paths is not None else BIOME_PATHS
        self.BPSBiomeStorePublisher_cls = client.symbols.objc_getClass("BPSBiomeStorePublisher")
        self.BMStoreConfig_cls = client.symbols.objc_getClass("BMStoreConfig")

    @property
    def streams(self) -> set[str]:
        """
        Scan all configured Biome paths for accessible streams.

        Any entries prefixed with `_DKEvent` are ignored.

        :return: Stream identifiers discovered under the configured paths.
        :rtype: set[str]
        """
        streams = set()
        for biome_path in self.biome_paths:
            if self.fs.accessible(biome_path):
                for stream in self.fs.listdir(biome_path):
                    if not stream.startswith("_DKEvent"):
                        streams.add(stream)
        return streams

    def resolve_stream_path(self, stream: str) -> Optional[RemotePath]:
        """
        Resolve a stream name to its first accessible `RemotePath`.

        :param str stream: Stream identifier.
        :return: Remote path of the stream, or `None` if not found.
        :rtype: Optional[RemotePath]
        """
        for biome_path in self.biome_paths:
            remote_path = self.fs.remote_path(f"{biome_path}/{stream}")
            if self.fs.accessible(remote_path):
                return remote_path
        return None

    def create_library(self) -> "BiomeLibrary":
        """
        Create a `BiomeLibrary` bound to this client.

        This library should either be deallocated (using `*.deallocate()`),
        or used as a context manager with a `with` statement.

        :return: Initialized Biome library.
        :rtype: BiomeLibrary
        """
        return BiomeLibrary(self._client, self)


class BiomeEvent:
    """
    Wrapper for a single Biome event extracted from a stream.

    :param DarwinSymbol raw_event: Raw Objective-C event object.
    :param str stream: Stream identifier.
    """

    def __init__(self, raw_event: DarwinSymbol, stream: str):
        self.native = raw_event
        self.stream = stream

    @cached_property
    def data(self) -> dict[str, Any]:
        """
        Event data.

        :return: Event data as a Python dict.
        :rtype: dict[str, Any]
        """
        return json.loads(self.native.objc_call("json").py())

    @cached_property
    def timestamp(self) -> datetime:
        """
        Event timestamp.

        :return: Event timestamp.
        :rtype: datetime
        """
        return datetime.fromtimestamp(self.data["eventTimestamp"])

    def __str__(self) -> str:
        """Human-readable string representation of the event data."""
        return str(self.data)

    def __repr__(self) -> str:
        """Debug representation of the event."""
        return f"<BiomeEvent from stream '{self.stream}'>"


class BiomeLibrary(Allocated):
    """
    Access to Biome streams and events.

    Resolve requested streams via the parent `Biome`, copies stream
    data to a temporary directory, and reads events using Biome store publishers.

    This object should either be deallocated (using `*.deallocate()`),
    or used as a context manager with a `with` statement.

    :param rpcclient.darwin.client.DarwinClient client: Connected Darwin client.
    :param Biome biome: Parent `Biome` instance.
    """

    def __init__(self, client: "DarwinClient", biome: Biome) -> None:
        Allocated.__init__(self)
        self._client = client
        self.biome = biome
        self.tmp_dir = self.biome.fs.remote_temp_dir()
        self.store_config = biome.BMStoreConfig_cls.objc_call("alloc").objc_call(
            "initWithStoreBasePath:segmentSize:", client.cf(str(self.tmp_dir)), 0x80000
        )
        try:
            self._BMEventClassForStreamIdentifier = client.symbols.BMEventClassForStreamIdentifier
        except SymbolAbsentError:
            self._BMEventClassForStreamIdentifier = None
        self._event_classes = {}

    @property
    def streams(self) -> set[str]:
        """
        Scan all configured Biome paths for accessible streams.

        Any entries prefixed with `_DKEvent` are ignored.

        :return: Stream identifiers discovered under the configured paths.
        :rtype: set[str]
        """
        return self.biome.streams

    def read_stream(self, stream: str, refresh: bool = True) -> list[BiomeEvent]:
        """
        Read events from a specified Biome stream.

        If *refresh* is `True`, copy the stream directory into `tmp_dir`
        before reading regurdless to the existance of a previously cached stream.
        Returns an empty list if the stream does not exist.

        :param str stream: Stream identifier.
        :param bool refresh: Whether to re-copy the stream before reading.
        :return: Parsed events.
        :rtype: list[BiomeEvent]
        """
        if refresh or not self.biome.fs.accessible(self.tmp_dir / stream):
            remote_path = self.biome.resolve_stream_path(stream)
            if remote_path is None:
                return []
            self.biome.fs.cp([remote_path], self.tmp_dir, recursive=True, force=True)
        return self._get_events_for_local_stream(stream)

    def _get_event_class(self, stream: str) -> DarwinSymbol:
        if stream in self._event_classes:
            return self._event_classes[stream]
        cf_stream_name = self._client.cf(stream)
        if self._BMEventClassForStreamIdentifier is not None:
            event_class = self._BMEventClassForStreamIdentifier(cf_stream_name)
        else:
            if stream.endswith(":tombstones"):
                return self._client.symbols.objc_getClass("BMTombstoneEvent")
            internal_library = self._client.symbols.BiomeLibraryAndInternalLibraryNode()
            stream_obj = internal_library.objc_call("streamWithIdentifier:error:", cf_stream_name, 0)
            event_class = (
                self._client.symbol(0)
                if stream_obj == 0
                else stream_obj.objc_call("configuration").objc_call("eventClass")
            )
        self._event_classes[stream] = event_class
        return event_class

    def _create_publisher(self, stream: str) -> DarwinSymbol:
        """
        Create and initialize a Biome store publisher for a given stream.

        :param str stream: Stream identifier.
        :return: Objective-C publisher object.
        :rtype: DarwinSymbol
        """
        cf_stream_name = self._client.cf(stream)
        event_class = self._get_event_class(stream)

        return self.biome.BPSBiomeStorePublisher_cls.objc_call("alloc").objc_call(
            "initWithStreamId:storeConfig:streamsAccessClient:eventDataClass:",
            cf_stream_name,
            self.store_config,
            0,
            event_class,
        )

    def _get_events_for_local_stream(self, stream: str) -> list[BiomeEvent]:
        """
        Read all available events for a local copy of the given stream.

        Assumes the stream data already exists under `tmp_dir`.

        :param str stream: Stream identifier.
        :return: Parsed events.
        :rtype: list[BiomeEvent]
        """
        publisher = self._create_publisher(stream)
        publisher.objc_call("startWithSubscriber:", 0)
        events: list[BiomeEvent] = []
        while publisher.objc_call("finished") != 1:
            event = publisher.objc_call("nextEvent")
            if event != 0:
                events.append(BiomeEvent(event, stream))
        return events

    def _deallocate(self) -> None:
        """
        Release allocated Objective-C objects and temporary resources.

        Called by `Allocated` during cleanup.
        """
        self.store_config.objc_call("release")
        self.tmp_dir.deallocate()



from contextlib import closing, suppress
from uuid import uuid4

import pytest

from rpcclient.client_manager import ClientManager
from rpcclient.clients.darwin.client import DarwinClient
from rpcclient.clients.ios.client import IosClient
from rpcclient.exceptions import BadReturnValueError
from rpcclient.protos.rpc_pb2 import ARCH_ARM64


@pytest.fixture
def client():
    with closing(ClientManager().create(hostname="127.0.0.1")) as c:
        yield c


@pytest.fixture
def tmp_path(client):
    tmp_path = "/tmp"
    with suppress(BadReturnValueError):
        tmp_path = client.fs.readlink(tmp_path)
    filename = client.fs.remote_path(tmp_path) / uuid4().hex
    client.fs.mkdir(filename, mode=0o777)
    try:
        yield filename
    finally:
        client.fs.remove(filename, recursive=True)


def pytest_addoption(parser):
    parser.addoption("--ci", action="store_true", default=False, help="Don't run local only tests")
    parser.addoption("--local-machine", action="store_true", default=False, help="Run local-machine tests")


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        """local_only: marks tests that require features the CI lacks (deselect with '-m "not local_only"')""",
    )
    config.addinivalue_line("markers", "darwin: marks tests that require darwin clients to run")
    config.addinivalue_line("markers", "ios: marks tests that require ios clients to run")
    config.addinivalue_line("markers", "local_machine: marks tests that require local_machine to run")
    config.addinivalue_line("markers", "arm: marks tests that require arm architecture to run")


def pytest_collection_modifyitems(config, items):
    skip_local_only = pytest.mark.skip(reason="remove --ci option to run")
    skip_not_darwin = pytest.mark.skip(reason="Darwin system is required for this test")
    skip_not_ios = pytest.mark.skip(reason="Ios system is required for this test")
    skip_not_arm = pytest.mark.skip(reason="Arm arch is required for this test")
    skip_not_local_machine = pytest.mark.skip(reason="Local machine is required for this test")

    with closing(ClientManager().create(hostname="127.0.0.1")) as c:
        is_darwin = isinstance(c, DarwinClient)
        is_ios = isinstance(c, IosClient)
        is_arm = c.arch == ARCH_ARM64

    for item in items:
        if "local_only" in item.keywords and config.getoption("--ci"):
            # --ci given in cli: skip local only tests.
            item.add_marker(skip_local_only)
        if "darwin" in item.keywords and not is_darwin:
            # Skip test that require Darwin on non Darwin system
            item.add_marker(skip_not_darwin)
        if "ios" in item.keywords and not is_ios:
            # Skip test that require ios
            item.add_marker(skip_not_ios)
        if "arm" in item.keywords and not is_arm:
            # Skip tests that require arm on non arm architecture
            item.add_marker(skip_not_arm)
        if "local_machine" in item.keywords and not config.getoption("--local-machine"):
            # Skip test that require local_machine
            item.add_marker(skip_not_local_machine)



import logging
from typing import TYPE_CHECKING

from rpcclient.exceptions import BadReturnValueError

if TYPE_CHECKING:
    from rpcclient.clients.ios.client import IosClient


class Backlight:
    """Display brightness controls."""

    def __init__(self, client: "IosClient") -> None:
        self._client = client

        BrightnessSystemClient = self._client.symbols.objc_getClass("BrightnessSystemClient")
        if not BrightnessSystemClient:
            logging.error("failed to load BrightnessSystemClient class")
        self._brightness = BrightnessSystemClient.objc_call("new")

    @property
    def brightness(self) -> float:
        """Return the display brightness in the range 0.0-1.0."""
        return self._brightness.objc_call("copyPropertyForKey:", self._client.cf("DisplayBrightness")).py()[
            "Brightness"
        ]

    @brightness.setter
    def brightness(self, value: float) -> None:
        """Set the display brightness in the range 0.0-1.0."""
        if not self._brightness.objc_call(
            "setProperty:forKey:", self._client.cf(value), self._client.cf("DisplayBrightness")
        ):
            raise BadReturnValueError("failed to set DisplayBrightness")



from typing import TYPE_CHECKING, Optional

from rpcclient.clients.darwin.structs import POLLIN, pollfd
from rpcclient.core.structs.consts import AF_UNIX, SOCK_STREAM
from rpcclient.core.subsystems.network import Socket

if TYPE_CHECKING:
    from rpcclient.core.client import CoreClient

FD_SIZE = 4
READ_SIZE = 0x10000


class CaptureFD:
    """
    Context manager, capturing output to a given `fd`. Read from it using the `read()` method.
    """

    def __init__(self, client: "CoreClient", fd: int, sock_buf_size: Optional[int] = None) -> None:
        """
        sock_buf_size is required for captures above 6KB, as any write above this value would block until a read is performed.

        :param rpcclient.client.client.Client client: Current client
        :param fd: FD to capture
        :param sock_buf_size: Buffer size for the capture socket, if not specified, default value is used.
        """
        self._client = client
        self.fd: int = fd
        self._backupfd: Optional[int] = None
        self._socket_pair: Optional[tuple[int, int]] = None
        self._sock_buf_size: Optional[int] = sock_buf_size

    def __enter__(self) -> "CaptureFD":
        with self._client.safe_malloc(FD_SIZE * 2) as socket_pair:
            socket_pair.item_size = FD_SIZE
            if self._client.symbols.socketpair(AF_UNIX, SOCK_STREAM, 0, socket_pair) != 0:
                self._client.raise_errno_exception("socketpair failed")
            self._socket_pair = (socket_pair[0].c_int32, socket_pair[1].c_int32)
        if self._sock_buf_size is not None:
            Socket(self._client, self._socket_pair[0]).setbufsize(self._sock_buf_size)
        self._backupfd = self._client.symbols.dup(self.fd).c_int32
        if self._backupfd == -1:
            self._backupfd = None
            self._client.raise_errno_exception("dup fd failed")
        if self._client.symbols.dup2(self._socket_pair[0], self.fd) < 0:
            self._client.raise_errno_exception("dup2 sock-fd failed")
        return self

    def __exit__(self, _type, value, traceback) -> None:
        if self._backupfd is not None:
            if self._client.symbols.dup2(self._backupfd, self.fd) < 0:
                self._client.raise_errno_exception("dup2 backup-fd failed")
            if self._client.symbols.close(self._backupfd) != 0:
                self._client.raise_errno_exception("close backupfd failed")
            self._backupfd = None
        if self._socket_pair is not None:
            if self._client.symbols.close(self._socket_pair[0]) != 0:
                self._client.raise_errno_exception(f"close _socket_pair[0] {self._socket_pair[0]} failed")
            if self._client.symbols.close(self._socket_pair[1]) != 0:
                self._client.raise_errno_exception(f"close _socket_pair[1] {self._socket_pair[1]} failed")
            self._socket_pair = None

    def read(self) -> bytes:
        """Read the bytes captured from `fd` so far."""
        data = b""
        if self._socket_pair is not None:
            with self._client.safe_malloc(READ_SIZE) as buff:
                read = READ_SIZE
                while read == READ_SIZE:
                    with self._client.safe_malloc(pollfd.sizeof()) as pfds:
                        pfds.poke(pollfd.build({"fd": self._socket_pair[1], "events": POLLIN, "revents": 0}))
                        if self._client.symbols.poll(pfds, 1, 0) != 1:
                            return data
                    read = self._client.symbols.read(self._socket_pair[1], buff, READ_SIZE).c_int32
                    if read == -1:
                        self._client.raise_errno_exception("read fd failed")
                    data += buff.peek(read)
        return data



import logging
import os
import subprocess
import time
import urllib.request
from socket import socket
from typing import Union

import requests

from rpcclient.exceptions import FailedToConnectError
from rpcclient.protocol.rpc_bridge import RpcBridge

logger = logging.getLogger(__name__)

DEFAULT_PORT = 5910
PROJECT_URL = "https://api.github.com/repos/doronz88/rpc-project/releases/latest"
BINARY_NAME = "rpcserver_macosx"


def create_tcp(
    *,
    hostname: Union[str, None] = None,
    host: Union[str, None] = None,
    port: int = DEFAULT_PORT,
    timeout: Union[float, None] = None,
) -> RpcBridge:
    """Connect via TCP and return a ProtoSocket."""
    target = hostname or host
    if not target:
        raise TypeError('tcp(): provide "hostname" or "host"')
    s = socket()
    if timeout is not None:
        s.settimeout(timeout)
    try:
        s.connect((target, port))
    except ConnectionRefusedError as e:
        s.close()
        raise FailedToConnectError() from e
    return RpcBridge.connect(s)


def create_local(
    *, project_url: str = PROJECT_URL, binary_name: str = BINARY_NAME, poll_interval: float = 0.1
) -> RpcBridge:
    """
    Download the latest rpcserver release asset, spawn it locally on a free port,
    and connect via TCP. Returns a ProtoSocket.
    """
    resp = requests.get(project_url)
    resp.raise_for_status()
    for asset in resp.json().get("assets", []):
        if asset.get("name") == binary_name:
            url = asset["browser_download_url"]
            logger.info("Downloading %s from %s", binary_name, url)
            urllib.request.urlretrieve(url, binary_name)
            os.chmod(binary_name, os.stat(binary_name).st_mode | 0o100)
            break

    # pick a free port
    srv = socket()
    srv.bind(("127.0.0.1", 0))
    port = srv.getsockname()[1]
    srv.close()

    # spawn
    subprocess.Popen([f"./{binary_name}", "-p", str(port)], cwd=os.getcwd())
    logger.info("rpcserver launched on port %d", port)

    # poll until connectable
    while True:
        try:
            return create_tcp(hostname="127.0.0.1", port=port)
        except FailedToConnectError:
            time.sleep(poll_interval)


def create_using_protocol(*, client, path: str) -> RpcBridge:
    if not hasattr(client, "create_worker") and callable(client.create_worker):
        raise ValueError("No existing client supports protocol worker creation.")
    return client.create_worker(path)



import sqlite3
from typing import TYPE_CHECKING

from rpcclient.core.structs.consts import SIGKILL
from rpcclient.core.subsystems.network import Network

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient

PINNING_RULED_DB = "/private/var/protected/trustd/pinningrules.sqlite3"
SYSTEM_CONFIGURATION_PLIST = "/private/var/Managed Preferences/mobile/com.apple.SystemConfiguration.plist"


class DarwinNetwork(Network):
    """ " Network utils"""

    def __init__(self, client: "DarwinClient"):
        super().__init__(client)

    @property
    def proxy_settings(self) -> dict:
        """Get proxy settings"""
        return self._client.symbols.CFNetworkCopySystemProxySettings().py()

    def set_http_proxy(self, ip: str, port: int) -> None:
        """Set http proxy"""
        with self._client.preferences.sc.open(SYSTEM_CONFIGURATION_PLIST) as config:
            config.set(
                "Proxies",
                {
                    "HTTPProxyType": 1,
                    "HTTPEnable": 1,
                    "HTTPPort": port,
                    "HTTPSProxy": ip,
                    "HTTPSPort": port,
                    "HTTPProxy": ip,
                    "HTTPSEnable": 1,
                    "BypassAllowed": 0,
                },
            )
        self._client.processes.get_by_basename("configd").kill(SIGKILL)

    def remove_http_proxy(self) -> None:
        """Remove http proxy settings"""
        with self._client.preferences.sc.open(SYSTEM_CONFIGURATION_PLIST) as config:
            config.remove("Proxies")
        self._client.processes.get_by_basename("configd").kill(SIGKILL)

    def remove_certificate_pinning(self) -> None:
        """Remove pinning rules from trustd"""
        with self._client.fs.remote_file(PINNING_RULED_DB) as local_db_file:
            # truncate pinning rules
            conn = sqlite3.connect(local_db_file)
            cursor = conn.cursor()
            cursor.execute("DELETE FROM rules")
            conn.commit()
            conn.close()

            # push new db
            self._client.fs.push(local_db_file, PINNING_RULED_DB, force=True)

        # restart trustd for changes to take affect
        self.kill_trustd()

    def restore_certificate_pinning(self) -> None:
        """Restore pinning rules from trustd"""
        if self._client.fs.accessible(PINNING_RULED_DB):
            self._client.fs.remove(PINNING_RULED_DB)

        # Upon startup trustd to reloads pinning rules from the last MobileAsset
        self.kill_trustd()

    def kill_trustd(self) -> None:
        """Kill trustd processes"""
        for process in self._client.processes.list():
            if process.basename == "trustd":
                process.kill(SIGKILL)

    def flush_dns(self) -> None:
        """Flush DNS cache"""
        self._client.processes.get_by_basename("mDNSResponder").kill(SIGKILL)



from contextlib import closing

from rpcclient.client_manager import ClientManager


def test_new_process_per_client(client):
    with closing(ClientManager().create(hostname="127.0.0.1")) as client2:
        assert client.pid != client2.pid
        client_process = client.processes.get_by_pid(client.pid)
        client2_process = client2.processes.get_by_pid(client2.pid)
        assert client_process.ppid == client2_process.ppid



import contextlib
import time
from enum import Enum
from typing import TYPE_CHECKING, Union

from rpcclient.clients.darwin.consts import (
    IOHIDDigitizerEventMask,
    IOHIDDigitizerTransducerType,
    IOHIDEventField,
    IOHIDEventFieldDigitizer,
    kCFAllocatorDefault,
    kHIDPage_Consumer,
    kHIDUsage_Csmr_ACSearch,
    kHIDUsage_Csmr_FastForward,
    kHIDUsage_Csmr_Menu,
    kHIDUsage_Csmr_Mute,
    kHIDUsage_Csmr_Pause,
    kHIDUsage_Csmr_Play,
    kHIDUsage_Csmr_PlayOrPause,
    kHIDUsage_Csmr_Power,
    kHIDUsage_Csmr_RandomPlay,
    kHIDUsage_Csmr_Repeat,
    kHIDUsage_Csmr_Rewind,
    kHIDUsage_Csmr_VolumeDecrement,
    kHIDUsage_Csmr_VolumeIncrement,
)
from rpcclient.exceptions import BadReturnValueError

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class TouchEventType(Enum):
    TOUCH_DOWN = 0
    TOUCH_UP = 1
    TOUCH_MOVE = 2


TOUCH_EVENT_SLEEP = 0.1


EVENT_TYPE_PARAMS = {
    TouchEventType.TOUCH_DOWN: {
        "event_flags": (
            IOHIDDigitizerEventMask.kIOHIDDigitizerEventAttribute.value
            | IOHIDDigitizerEventMask.kIOHIDDigitizerEventTouch.value
            | IOHIDDigitizerEventMask.kIOHIDDigitizerEventIdentity.value
        ),
        "button_mask": 1,
        "touch": True,
    },
    TouchEventType.TOUCH_MOVE: {
        "event_flags": (
            IOHIDDigitizerEventMask.kIOHIDDigitizerEventPosition.value
            | IOHIDDigitizerEventMask.kIOHIDDigitizerEventAttribute.value
        ),
        "button_mask": 1,
        "touch": True,
    },
    TouchEventType.TOUCH_UP: {
        "event_flags": (
            IOHIDDigitizerEventMask.kIOHIDDigitizerEventAttribute.value
            | IOHIDDigitizerEventMask.kIOHIDDigitizerEventTouch.value
            | IOHIDDigitizerEventMask.kIOHIDDigitizerEventIdentity.value
        ),
        "button_mask": 0,
        "touch": False,
    },
}


class Hid:
    """Control HID devices and simulate events"""

    def __init__(self, client: "DarwinClient"):
        self._client = client

    def send_double_home_button_press(self):
        self.send_home_button_press()
        self.send_home_button_press()

    def send_rewind_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Rewind)

    def send_random_play_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_RandomPlay)

    def send_repeat_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Repeat)

    def send_fast_forward_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_FastForward)

    def send_play_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Play)

    def send_pause_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Pause)

    def send_play_pause_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_PlayOrPause)

    def send_search_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_ACSearch)

    def send_mute_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Mute)

    def send_home_button_press(self):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Menu)

    def send_power_button_press(self, interval: int = 0):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_Power, interval=interval)

    def send_volume_down_button_press(self, interval: int = 0):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_VolumeDecrement, interval=interval)

    def send_volume_up_button_press(self, interval: int = 0):
        self.send_key_press(kHIDPage_Consumer, kHIDUsage_Csmr_VolumeIncrement, interval=interval)

    def send_key_press(self, page: int, key_code: int, interval: Union[float, int] = 0):
        self.send_keyboard_event(page, key_code, True)
        if interval:
            time.sleep(interval)
        self.send_keyboard_event(page, key_code, False)

    def send_keyboard_event(self, page: int, key_code: int, down: bool):
        event = self._client.symbols.IOHIDEventCreateKeyboardEvent(
            kCFAllocatorDefault, self._client.symbols.mach_absolute_time(), page, key_code, down, 0
        )
        if not event:
            raise BadReturnValueError("IOHIDEventCreateKeyboardEvent() failed")

        self.dispatch(event)

    def send_swipe_right(self):
        self.send_swipe(0.5, 0.5, 1.0, 0.5)

    def send_swipe_left(self):
        self.send_swipe(0.5, 0.5, 0.0, 0.5)

    def send_swipe_up(self):
        self.send_swipe(0.5, 1.5, 0.5, 0.5)

    def send_swipe_down(self):
        self.send_swipe(0.5, 0.5, 0.5, 1.5)

    def send_swipe(self, from_x: float, from_y: float, to_x: float, to_y: float):
        self.send_touch_event(TouchEventType.TOUCH_DOWN, from_x, from_y)
        time.sleep(TOUCH_EVENT_SLEEP)
        self.send_touch_event(TouchEventType.TOUCH_MOVE, to_x, to_y)
        self.send_touch_event(TouchEventType.TOUCH_UP, to_x, to_y)

    def send_touch_event(self, event_type: TouchEventType, x: float, y: float, pressure: float = 0.4):
        params = EVENT_TYPE_PARAMS[event_type]

        timestamp = self._client.symbols.mach_absolute_time()

        event_flags = params["event_flags"]
        touch = params["touch"]
        button_mask = params["button_mask"]

        parent = self.create_digitizer_event(
            IOHIDDigitizerTransducerType.kIOHIDDigitizerTransducerTypeHand,
            0,
            0,
            event_flags,
            0,
            x,
            y,
            0.0,
            0.0,
            0.0,
            touch,
            touch,
            0,
            timestamp=timestamp,
        )

        child = self.create_digitizer_finger_event(
            2, 2, event_flags, button_mask, x, y, 0.0, pressure, 0.0, touch, touch, 0, timestamp=timestamp
        )
        self._client.symbols.IOHIDEventAppendEvent(parent, child)
        self.dispatch(parent)

    def create_digitizer_event(
        self,
        type_: Union[IOHIDDigitizerTransducerType, int],
        index: int,
        identity: int,
        event_mask: int,
        button_mask: int,
        x: float,
        y: float,
        z: float,
        tip_pressure: float,
        barrel_pressure: float,
        range_: bool,
        touch: bool,
        options: int,
        timestamp=None,
    ):
        if timestamp is None:
            timestamp = self._client.symbols.mach_absolute_time()

        event = self._client.symbols.IOHIDEventCreateDigitizerEvent(
            kCFAllocatorDefault,
            timestamp,
            type_,
            index,
            identity,
            event_mask,
            button_mask,
            x,
            y,
            z,
            tip_pressure,
            barrel_pressure,
            range_,
            touch,
            options,
        )

        if not event:
            raise BadReturnValueError("IOHIDEventCreateDigitizerEvent() failed")

        self._client.symbols.IOHIDEventSetIntegerValue(event, IOHIDEventField.kIOHIDEventFieldIsBuiltIn, 0)
        self._client.symbols.IOHIDEventSetIntegerValue(
            event, IOHIDEventFieldDigitizer.kIOHIDEventFieldDigitizerIsDisplayIntegrated, 1
        )
        self._client.symbols.IOHIDEventSetSenderID(event, 0x8000000817319375)

        return event

    def create_digitizer_finger_event(
        self,
        index: int,
        identity: int,
        event_mask: int,
        button_mask: int,
        x: float,
        y: float,
        z: float,
        tip_pressure: float,
        twist: float,
        range_: bool,
        touch: bool,
        options: int,
        timestamp=None,
    ):
        if timestamp is None:
            timestamp = self._client.symbols.mach_absolute_time()

        event = self._client.symbols.IOHIDEventCreateDigitizerFingerEvent(
            kCFAllocatorDefault,
            timestamp,
            index,
            identity,
            event_mask,
            button_mask,
            x,
            y,
            z,
            tip_pressure,
            twist,
            range_,
            touch,
            options,
        )

        if not event:
            raise BadReturnValueError("IOHIDEventCreateDigitizerFingerEvent() failed")

        self._client.symbols.IOHIDEventSetFloatValue(
            event, IOHIDEventFieldDigitizer.kIOHIDEventFieldDigitizerMajorRadius, 0.5
        )
        return event

    @contextlib.contextmanager
    def create_hid_client(self):
        client = self._client.symbols.IOHIDEventSystemClientCreate(0)

        if not client:
            raise BadReturnValueError("IOHIDEventSystemClientCreate() failed")

        try:
            yield client
        finally:
            self._client.symbols.CFRelease(client)

    def dispatch(self, event):
        with self.create_hid_client() as hid_client:
            self._client.symbols.IOHIDEventSystemClientDispatchEvent(hid_client, event)



from typing import Any

from IPython import get_ipython
from IPython.terminal.prompts import Prompts, Token


class RpcPrompt(Prompts):
    """Custom IPython prompt that reflects the active RPC client (global `p`)."""

    def in_prompt_tokens(self) -> list[tuple[Any, str]]:
        """Build input prompt tokens."""
        return self._build_tokens(Token.Prompt, Token.PromptNum)

    def out_prompt_tokens(self) -> list[tuple[Any, str]]:
        """Build output prompt tokens."""
        return self._build_tokens(Token.OutPrompt, Token.OutPromptNum)

    @staticmethod
    def _build_tokens(base_token: Any, num_token: Any) -> list[tuple[Any, str]]:
        """Return token list for the prompt, falling back if `p` is missing."""
        try:
            client = get_ipython().user_ns.get("p")
            result = [
                (base_token, f"[{client.platform}| "),
                (num_token, f"({client.id}) "),
                (base_token, f"{client.progname}]: "),
            ]
        except AttributeError:
            result = [(base_token, "[Rpc-client]: ")]
        return result



from abc import abstractmethod


class Allocated:
    """resource allocated on remote host that needs to be free"""

    def __init__(self):
        self._deallocated = False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.deallocate()

    @abstractmethod
    def _deallocate(self):
        pass

    def deallocate(self):
        if not self._deallocated:
            self._deallocated = True
            self._deallocate()



import logging
import threading
from typing import Any, Callable, Generic, Optional, TypeVar

logger = logging.getLogger(__name__)

E = TypeVar("E")
Callback = Callable[..., Any]


class EventNotifier(Generic[E]):
    """Notifies registered callbacks when events occur"""

    def __init__(self) -> None:
        self._lock = threading.RLock()
        self._data: dict[E, set[Callback]] = {}

    def register(self, event: E, callback: Callback) -> None:
        """Register a callback for an event"""
        if not callable(callback):
            raise TypeError("callback must be callable")
        with self._lock:
            self._data.setdefault(event, set()).add(callback)

    def unregister(self, event: E, callback: Callback) -> bool:
        """Unregister a callback; returns True if it was present"""
        with self._lock:
            s = self._data.get(event)
            if not s:
                return False
            try:
                s.remove(callback)
                if not s:
                    self._data.pop(event, None)
            except KeyError:
                return False
            return True

    def register_once(self, event: E, callback: Callback) -> None:
        """Register a callback that will be invoked at most once"""

        def _wrapper(*args: Any, **kwargs: Any) -> None:
            try:
                callback(*args, **kwargs)
            finally:
                # ensure we remove even if callback raises
                self.unregister(event, _wrapper)

        self.register(event, _wrapper)

    def clear(self, event: Optional[E] = None) -> None:
        """Remove all callbacks (optionally only for one event)"""
        with self._lock:
            if event is None:
                self._data.clear()
            else:
                self._data.pop(event, None)

    def listeners(self, event: E) -> tuple[Callback, ...]:
        """Return current listeners for an event (snapshot)"""
        with self._lock:
            return tuple(self._data.get(event, ()))

    def has_listeners(self, event: E) -> bool:
        with self._lock:
            s = self._data.get(event)
            return bool(s)

    def notify(self, event: E, *args: Any, **kwargs: Any) -> None:
        """Invoke all callbacks registered for the given event"""
        with self._lock:
            callbacks = self._data.get(event, set()).copy()

        for callback in callbacks:
            try:
                callback(*args, **kwargs)
            except Exception:
                logger.error(
                    "Error in callback %s for event %r",
                    getattr(callback, "__name__", repr(callback)),
                    event,
                    exc_info=True,
                )



import socket as pysock
import typing
from collections import namedtuple
from typing import TYPE_CHECKING, Optional

from rpcclient.clients.darwin.structs import timeval
from rpcclient.core.allocated import Allocated
from rpcclient.core.structs.consts import (
    AF_INET,
    AF_INET6,
    AF_UNIX,
    EPIPE,
    F_GETFL,
    F_SETFL,
    MSG_NOSIGNAL,
    O_NONBLOCK,
    SO_RCVBUF,
    SO_RCVTIMEO,
    SO_SNDBUF,
    SO_SNDTIMEO,
    SOCK_STREAM,
    SOL_SOCKET,
)
from rpcclient.core.structs.generic import hostent, ifaddrs, sockaddr, sockaddr_in, sockaddr_in6, sockaddr_un, uint32_t
from rpcclient.core.symbol import Symbol
from rpcclient.exceptions import BadReturnValueError

if TYPE_CHECKING:
    from rpcclient.core.client import CoreClient

Interface = namedtuple("Interface", "name address netmask broadcast")
Hostentry = namedtuple("Hostentry", "name aliases addresses")


class Socket(Allocated):
    CHUNK_SIZE = 1024

    def __init__(self, client: "CoreClient", fd: int):
        """
        :param rpcclient.client.client.Client client:
        :param fd:
        """
        super().__init__()
        self._client = client
        self.fd = fd
        self._blocking = self._getblocking()
        self._timeout = None

    def _deallocate(self):
        """Close the remote socket file descriptor."""
        fd = self._client.symbols.close(self.fd).c_int32
        if fd < 0:
            raise BadReturnValueError(f"failed to close fd: {fd}")

    def send(self, buf: typing.Union[bytes, Symbol], size: Optional[int] = None, flags: int = 0) -> int:
        """
        Send bytes from buf on the remote socket fd.

        :param buf: buffer to send
        :param size: If None, use len(buf)
        :param flags: flags for send() syscall. MSG_NOSIGNAL will always be added
        :return: how many bytes were sent
        """
        if size is None:
            if isinstance(buf, Symbol):
                raise ValueError("cannot calculate size argument for Symbol objects")
            size = len(buf)
        n = self._client.symbols.send(self.fd, buf, size, MSG_NOSIGNAL | flags).c_int64
        if n < 0:
            if self._client.errno == EPIPE:
                self.deallocate()
            self._client.raise_errno_exception(f"failed to send on fd: {self.fd}")
        return n

    def sendall(self, buf: bytes, flags: int = 0) -> None:
        """Send the entire buffer, retrying until all bytes are written."""
        size = len(buf)
        offset = 0
        with self._client.safe_malloc(size) as block:
            block.poke(buf)
            while offset < size:
                sent = self.send(block + offset, size - offset, flags=flags)
                offset += sent

    def _recv(self, chunk: Symbol, size: int, flags: int = 0) -> bytes:
        err = self._client.symbols.recv(self.fd, chunk, size, MSG_NOSIGNAL | flags).c_int64
        if err < 0:
            self._client.raise_errno_exception(f"recv() failed for fd: {self.fd}")
        elif err == 0:
            self._client.raise_errno_exception(f"recv() failed for fd: {self.fd} (peer closed)")
        return chunk.peek(err)

    def recv(self, size: int = CHUNK_SIZE, flags: int = 0) -> bytes:
        """
        Receive up to size bytes from the remote socket fd.

        :param size: chunk size
        :param flags: flags for recv() syscall. MSG_NOSIGNAL will always be added
        :return: received bytes
        """
        with self._client.safe_malloc(size) as chunk:
            return self._recv(chunk, size, flags=flags)

    def recvall(self, size: int, flags: int = 0) -> bytes:
        """recv at remote until all buffer is received"""
        buf = b""
        with self._client.safe_malloc(size) as chunk:
            while len(buf) < size:
                buf += self._recv(chunk, size - len(buf), flags=flags)
        return buf

    def setsockopt(self, level: int, option_name: int, option_value: bytes):
        if self._client.symbols.setsockopt(self.fd, level, option_name, option_value, len(option_value)) != 0:
            self._client.raise_errno_exception(f"setsockopt() failed: {self._client.last_error}")

    def setbufsize(self, size: int) -> None:
        self.setsockopt(SOL_SOCKET, SO_SNDBUF, uint32_t.build(size))
        self.setsockopt(SOL_SOCKET, SO_RCVBUF, uint32_t.build(size))

    def settimeout(self, seconds: int):
        self._timeout = seconds
        self.setsockopt(SOL_SOCKET, SO_RCVTIMEO, timeval.build({"tv_sec": seconds, "tv_usec": 0}))
        self.setsockopt(SOL_SOCKET, SO_SNDTIMEO, timeval.build({"tv_sec": seconds, "tv_usec": 0}))

    def gettimeout(self) -> typing.Optional[int]:
        return self._timeout

    def setblocking(self, blocking: bool):
        opts = self._client.symbols.fcntl(self.fd, F_GETFL, 0).c_uint64
        if not blocking:
            opts |= O_NONBLOCK
        else:
            opts &= ~O_NONBLOCK
        if self._client.symbols.fcntl(self.fd, F_SETFL, opts) != 0:
            self._client.raise_errno_exception(f"fcntl() failed: {self._client.last_error}")
        self._blocking = blocking

    def getblocking(self) -> bool:
        return self._blocking

    def _getblocking(self) -> bool:
        return not bool(self._client.symbols.fcntl(self.fd, F_GETFL, 0) & O_NONBLOCK)

    def __repr__(self):
        return f"<{self.__class__.__name__} FD:{self.fd} BLOCKING:{self._blocking}>"


class Network:
    def __init__(self, client: "CoreClient"):
        """
        :param rpcclient.client.client.Client client:
        """
        self._client = client

    def socket(self, family=AF_INET, socktype=SOCK_STREAM, proto=0) -> int:
        """Create a remote socket and return its file descriptor."""
        result = self._client.symbols.socket(family, socktype, proto).c_int64
        if result == 0:
            self._client.raise_errno_exception(f"failed to create socket: {result}")
        return result

    def tcp_connect(self, address: str, port: int) -> Socket:
        """make target connect to given address:port and get socket object"""
        family = AF_INET6 if ":" in address else AF_INET
        sockfd = self.socket(family=family, socktype=SOCK_STREAM, proto=0)
        if family == AF_INET:
            servaddr = sockaddr_in.build({"sin_addr": pysock.inet_pton(family, address), "sin_port": port})
        else:
            servaddr = sockaddr_in6.build({"sin6_addr": pysock.inet_pton(family, address), "sin6_port": port})
        self._client.errno = 0
        error = self._client.symbols.connect(sockfd, servaddr, len(servaddr)).c_int64
        if error == -1:
            self._client.symbols.close(sockfd)
            self._client.raise_errno_exception(f"failed connecting to: {address}:{port}")
        return Socket(self._client, sockfd)

    def unix_connect(self, filename: str) -> Socket:
        """make target connect to given unix path and get socket object"""
        sockfd = self.socket(family=AF_UNIX, socktype=SOCK_STREAM, proto=0)
        servaddr = sockaddr_un.build({"sun_path": filename})
        self._client.errno = 0
        error = self._client.symbols.connect(sockfd, servaddr, len(servaddr)).c_int64
        if error == -1:
            self._client.symbols.close(sockfd)
            self._client.raise_errno_exception(f"failed connecting to: {filename}")
        return Socket(self._client, sockfd)

    def gethostbyname(self, name: str) -> Optional[Hostentry]:
        """Query DNS record. Returns None if not found."""
        aliases = []
        addresses = []
        result = self._client.symbols.gethostbyname(name)
        if result == 0:
            return None
        result = hostent(self._client).parse_stream(result)
        p_aliases = result.h_aliases

        i = 0
        while p_aliases[i]:
            aliases.append(p_aliases[i].peek_str())
            i += 1

        addr_list = result.h_addr_list

        i = 0
        while addr_list[i]:
            addresses.append(pysock.inet_ntoa(addr_list[i].peek(4)))
            i += 1

        return Hostentry(name=result.h_name, aliases=aliases, addresses=addresses)

    @property
    def interfaces(self) -> list[Interface]:
        """get current interfaces"""
        results = []
        my_ifaddrs = ifaddrs(self._client)
        with self._client.safe_calloc(8) as addresses:
            if self._client.symbols.getifaddrs(addresses).c_int64 < 0:
                self._client.raise_errno_exception("getifaddrs failed")

            current = my_ifaddrs.parse_stream(addresses[0])

            while current:
                family = sockaddr.parse(current.ifa_addr.peek(sockaddr.sizeof())).sa_family

                if family == AF_INET:
                    address = pysock.inet_ntoa(sockaddr_in.parse_stream(current.ifa_addr).sin_addr)
                    netmask = pysock.inet_ntoa(sockaddr_in.parse_stream(current.ifa_netmask).sin_addr)
                    broadcast = pysock.inet_ntoa(sockaddr_in.parse_stream(current.ifa_dstaddr).sin_addr)
                    results.append(
                        Interface(name=current.ifa_name, address=address, netmask=netmask, broadcast=broadcast)
                    )

                current = current.ifa_next

            self._client.symbols.freeifaddrs(addresses[0])
        return results



from pathlib import Path
from typing import TYPE_CHECKING

from pycrashreport.crash_report import CrashReportBase, get_crash_report_from_buf

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class CrashReports:
    """ " manage crash reports"""

    def __init__(self, client: "DarwinClient", crash_reports_dir):
        self._client = client
        self._crash_reports_dir = crash_reports_dir

    def set_symbolicated(self, enabled: bool = True):
        """
        enable/disable crash reports symbolication
        https://github.com/dlevi309/Symbolicator
        """
        self._client.preferences.cf.set("SymbolicateCrashes", enabled, "com.apple.CrashReporter", "root")

        # bugfix: at some point, this setting was moved to "com.apple.osanalytics" bundle identifier
        self._client.preferences.cf.set("SymbolicateCrashes", enabled, "com.apple.osanalytics", "root")

    def list(self, prefixed="") -> list[CrashReportBase]:
        """get a list of all crash reports as CrashReport parsed objects"""
        result = []
        for root in self._client.roots:
            root = Path(root) / self._crash_reports_dir

            if not self._client.fs.accessible(root):
                continue

            for entry in self._client.fs.scandir(root):
                if entry.is_file() and entry.name.endswith(".ips") and entry.name.startswith(prefixed):
                    with self._client.fs.open(entry.path, "r") as f:
                        result.append(get_crash_report_from_buf(f.read().decode(), filename=entry.path))
        return result

    def clear(self, prefixed=""):
        """remove all existing crash reports"""
        for entry in self.list(prefixed=prefixed):
            self._client.fs.remove(entry.filename)



import pytest

from rpcclient.clients.darwin.consts import NSStringEncoding
from rpcclient.clients.darwin.symbol import DarwinSymbol

pytestmark = pytest.mark.darwin


def test_method_by_method_name(client):
    NSString = client.objc_get_class("NSString")
    ascii_encoding = NSStringEncoding.NSASCIIStringEncoding
    str1 = NSString.stringWithCString_encoding_("Taylor Swift", ascii_encoding).objc_symbol
    assert str1.cStringUsingEncoding_(ascii_encoding).peek_str() == "Taylor Swift"
    assert str1.length == len("Taylor Swift")
    assert str1.lowercaseString().objc_symbol.cStringUsingEncoding_(ascii_encoding).peek_str() == "taylor swift"
    assert str1.uppercaseString().objc_symbol.cStringUsingEncoding_(ascii_encoding).peek_str() == "TAYLOR SWIFT"


def test_calling_property(client):
    d = client.symbols.objc_getClass("NSMutableDictionary").objc_call("new")
    # call method
    d.objc_call("setObject:forKey:", client.cf("value"), client.cf("key"))
    # call property
    assert d.objc_symbol.description.py() == "{\n    key = value;\n}"


def test_set_implementation(client):
    pid = client.symbols.getpid()

    client.objc_get_class("NSJSONSerialization").get_method("isValidJSONObject:").set_implementation(
        client.symbols.getpid
    )
    assert client.objc_get_class("NSJSONSerialization").isValidJSONObject_() == pid


@pytest.mark.parametrize("value", [True, False])
def test_always_return(client, value):
    client.objc_get_class("NSJSONSerialization").get_method("isValidJSONObject:").always_return(value)
    assert client.objc_get_class("NSJSONSerialization").isValidJSONObject_() == value


def test_ivar_symbol(client):
    NSString = client.objc_get_class("NSString")
    ascii_encoding = NSStringEncoding.NSASCIIStringEncoding
    str1 = NSString.stringWithCString_encoding_("Taylor Swift", ascii_encoding).objc_symbol
    assert isinstance(str1.isa, DarwinSymbol)



import logging
from typing import Optional

import click
import coloredlogs

from rpcclient.client_manager import ClientManager
from rpcclient.console.console import Console, disable_loggers
from rpcclient.transports import DEFAULT_PORT

coloredlogs.install(level=logging.DEBUG)

disable_loggers()

startup_files_option = click.option(
    "-f",
    "--startup-files",
    type=click.Path(exists=True),
    multiple=True,
    help="File(s) (python) to run on session start. Multiple files can be provided.",
)


@click.command()
@startup_files_option
def rpclocal(startup_files: tuple[str]) -> None:
    """connect to a local machine"""
    manager = ClientManager()
    client = manager.create(mode="local")

    Console(manager).interactive(switch_cid=client.id, startup_files=startup_files)


@click.command()
@click.argument("hostname", required=False)
@click.option("-p", "--port", type=click.INT, default=DEFAULT_PORT, help="TCP port to connect to")
@click.option("-r", "--rebind-symbols", is_flag=True, help="reload all symbols upon connection")
@click.option("-l", "--load-all-libraries", is_flag=True, help="load all libraries")
@startup_files_option
def rpcclient(
    hostname: Optional[str], port: int, rebind_symbols: bool, load_all_libraries: bool, startup_files: tuple[str]
):
    """
    Start the console.
    If HOSTNAME is provided, connect immediately.
    Otherwise, start without a connection.
    You can connect later from the console.
    """
    manager = ClientManager()
    cid = None
    if hostname:
        client = manager.create(hostname=hostname, port=port)
        cid = client.id
        if rebind_symbols:
            client.rebind_symbols()
        if load_all_libraries:
            client.load_all_libraries()

    Console(manager).interactive(switch_cid=cid, startup_files=startup_files)


if __name__ == "__main__":
    rpcclient()



import threading
from enum import Enum, auto
from typing import Generic, Optional, TypeVar

from rpcclient.event_notifier import EventNotifier

K = TypeVar("K")
V = TypeVar("V")


class RegistryEvent(Enum):
    """Event identifiers for registry operations."""

    REGISTERED = auto()
    UNREGISTERED = auto()
    CLEARED = auto()


class Registry(Generic[K, V]):
    """Thread-safe 1:1 registry (key -> value)."""

    def __init__(self, initial_data: Optional[dict[K, V]] = None, notifier: Optional[EventNotifier] = None) -> None:
        self._lock = threading.RLock()
        self._data: dict[K, V] = {}
        self.notifier: EventNotifier = notifier or EventNotifier()

        if initial_data is not None:
            self.update(initial_data)

    def register(self, key: K, value: V, overwrite: bool = False, internal: bool = False) -> None:
        """Add or replace a value for a key."""
        if key in self._data and not overwrite:
            raise KeyError(f"key already exists: {key}, use overwrite=True to overwrite")
        with self._lock:
            self._data[key] = value
        self.notifier.notify(RegistryEvent.REGISTERED, *(key, value), internal=internal)

    def unregister(self, key: K) -> None:
        """Remove the key"""
        with self._lock:
            self._data.pop(key, None)
        self.notifier.notify(RegistryEvent.UNREGISTERED, key)

    def update(self, entries: dict[K, V], overwrite: bool = False) -> None:
        for key, value in entries.items():
            self.register(key, value, overwrite)

    def clone(self) -> "Registry":
        copied = Registry()
        copied.update(dict(self.items()), overwrite=True)
        return copied

    def clear(self) -> None:
        """Remove all entries."""
        with self._lock:
            self._data.clear()
        self.notifier.notify(RegistryEvent.CLEARED)

    def get(self, key: K) -> Optional[V]:
        """Get value for a key, or None if missing."""
        with self._lock:
            return self._data.get(key)

    def items(self) -> list[tuple[K, V]]:
        """Snapshot of all (key, value) pairs."""
        with self._lock:
            return sorted(self._data.items())

    def __contains__(self, key: K) -> bool:
        """Return True if the key exists."""
        with self._lock:
            return key in self._data

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.items()!r})"

    def __str__(self) -> str:
        items = self.items()
        lines = [f"{self.__class__.__name__} ({len(items)} entries):"]
        lines += [f"  {k!r}: {v!r}" for k, v in items]
        return "\n".join(lines)

    def __len__(self):
        return len(self._data)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

class _MsgId:
    ValueType = _typing.NewType("ValueType", _builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _MsgIdEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[_MsgId.ValueType], _builtins.type):
    DESCRIPTOR: _descriptor.EnumDescriptor
    MSG_ID_UNKNOWN: _MsgId.ValueType  # 0
    REQ_DLOPEN: _MsgId.ValueType  # 1
    REQ_DLCLOSE: _MsgId.ValueType  # 2
    REQ_DLSYM: _MsgId.ValueType  # 3
    REQ_PEEK: _MsgId.ValueType  # 4
    REQ_POKE: _MsgId.ValueType  # 5
    REQ_EXEC: _MsgId.ValueType  # 6
    REQ_CALL: _MsgId.ValueType  # 7
    REQ_CLOSE_CLIENT: _MsgId.ValueType  # 8
    REQ_LIST_DIR: _MsgId.ValueType  # 9
    REQ_DUMMY_BLOCK: _MsgId.ValueType  # 10
    REQ_GET_CLASS_LIST: _MsgId.ValueType  # 11
    REQ_SHOW_OBJECT: _MsgId.ValueType  # 12
    REQ_SHOW_CLASS: _MsgId.ValueType  # 13

class MsgId(_MsgId, metaclass=_MsgIdEnumTypeWrapper): ...

MSG_ID_UNKNOWN: MsgId.ValueType  # 0
REQ_DLOPEN: MsgId.ValueType  # 1
REQ_DLCLOSE: MsgId.ValueType  # 2
REQ_DLSYM: MsgId.ValueType  # 3
REQ_PEEK: MsgId.ValueType  # 4
REQ_POKE: MsgId.ValueType  # 5
REQ_EXEC: MsgId.ValueType  # 6
REQ_CALL: MsgId.ValueType  # 7
REQ_CLOSE_CLIENT: MsgId.ValueType  # 8
REQ_LIST_DIR: MsgId.ValueType  # 9
REQ_DUMMY_BLOCK: MsgId.ValueType  # 10
REQ_GET_CLASS_LIST: MsgId.ValueType  # 11
REQ_SHOW_OBJECT: MsgId.ValueType  # 12
REQ_SHOW_CLASS: MsgId.ValueType  # 13
Global___MsgId: _TypeAlias = MsgId  # noqa: Y015

@_typing.final
class RequestShowObject(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestShowObject: _TypeAlias = RequestShowObject  # noqa: Y015

@_typing.final
class ReplyShowObject(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: _builtins.int
    description: _builtins.str
    def __init__(
        self,
        *,
        description: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["description", b"description"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyShowObject: _TypeAlias = ReplyShowObject  # noqa: Y015

@_typing.final
class RequestShowClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestShowClass: _TypeAlias = RequestShowClass  # noqa: Y015

@_typing.final
class ReplyShowClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: _builtins.int
    description: _builtins.str
    def __init__(
        self,
        *,
        description: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["description", b"description"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyShowClass: _TypeAlias = ReplyShowClass  # noqa: Y015

@_typing.final
class RequestDlopen(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    FILENAME_FIELD_NUMBER: _builtins.int
    MODE_FIELD_NUMBER: _builtins.int
    filename: _builtins.str
    mode: _builtins.int
    def __init__(
        self,
        *,
        filename: _builtins.str = ...,
        mode: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["filename", b"filename", "mode", b"mode"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlopen: _TypeAlias = RequestDlopen  # noqa: Y015

@_typing.final
class ReplyDlopen(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlopen: _TypeAlias = ReplyDlopen  # noqa: Y015

@_typing.final
class RequestDlclose(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlclose: _TypeAlias = RequestDlclose  # noqa: Y015

@_typing.final
class ReplyDlclose(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    RES_FIELD_NUMBER: _builtins.int
    res: _builtins.int
    def __init__(
        self,
        *,
        res: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["res", b"res"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlclose: _TypeAlias = ReplyDlclose  # noqa: Y015

@_typing.final
class RequestDlsym(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    SYMBOL_NAME_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    symbol_name: _builtins.str
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
        symbol_name: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle", "symbol_name", b"symbol_name"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlsym: _TypeAlias = RequestDlsym  # noqa: Y015

@_typing.final
class ReplyDlsym(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PTR_FIELD_NUMBER: _builtins.int
    ptr: _builtins.int
    def __init__(
        self,
        *,
        ptr: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["ptr", b"ptr"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlsym: _TypeAlias = ReplyDlsym  # noqa: Y015

@_typing.final
class RequestExec(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    BACKGROUND_FIELD_NUMBER: _builtins.int
    ARGV_FIELD_NUMBER: _builtins.int
    ENVP_FIELD_NUMBER: _builtins.int
    background: _builtins.bool
    @_builtins.property
    def argv(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    @_builtins.property
    def envp(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(
        self,
        *,
        background: _builtins.bool = ...,
        argv: _abc.Iterable[_builtins.str] | None = ...,
        envp: _abc.Iterable[_builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["argv", b"argv", "background", b"background", "envp", b"envp"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestExec: _TypeAlias = RequestExec  # noqa: Y015

@_typing.final
class ReplyExec(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PID_FIELD_NUMBER: _builtins.int
    FD_FIELD_NUMBER: _builtins.int
    pid: _builtins.int
    fd: _builtins.int
    def __init__(
        self,
        *,
        pid: _builtins.int = ...,
        fd: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["fd", b"fd", "pid", b"pid"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyExec: _TypeAlias = ReplyExec  # noqa: Y015

@_typing.final
class RequestCall(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    VA_LIST_INDEX_FIELD_NUMBER: _builtins.int
    ARGV_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    va_list_index: _builtins.int
    @_builtins.property
    def argv(self) -> _containers.RepeatedCompositeFieldContainer[Global___Argument]: ...
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        va_list_index: _builtins.int = ...,
        argv: _abc.Iterable[Global___Argument] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "argv", b"argv", "va_list_index", b"va_list_index"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestCall: _TypeAlias = RequestCall  # noqa: Y015

@_typing.final
class ReplyCall(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ARM_REGISTERS_FIELD_NUMBER: _builtins.int
    RETURN_VALUE_FIELD_NUMBER: _builtins.int
    return_value: _builtins.int
    @_builtins.property
    def arm_registers(self) -> Global___ReturnRegistersArm: ...
    def __init__(
        self,
        *,
        arm_registers: Global___ReturnRegistersArm | None = ...,
        return_value: _builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_return_values: _TypeAlias = _typing.Literal["arm_registers", "return_value"]  # noqa: Y015
    _WhichOneofArgType_return_values: _TypeAlias = _typing.Literal["return_values", b"return_values"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_return_values) -> _WhichOneofReturnType_return_values | None: ...

Global___ReplyCall: _TypeAlias = ReplyCall  # noqa: Y015

@_typing.final
class RequestPeek(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    size: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        size: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestPeek: _TypeAlias = RequestPeek  # noqa: Y015

@_typing.final
class ReplyPeek(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    data: _builtins.bytes
    def __init__(
        self,
        *,
        data: _builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyPeek: _TypeAlias = ReplyPeek  # noqa: Y015

@_typing.final
class RequestPoke(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    DATA_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    data: _builtins.bytes
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        data: _builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestPoke: _TypeAlias = RequestPoke  # noqa: Y015

@_typing.final
class ReplyPoke(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___ReplyPoke: _TypeAlias = ReplyPoke  # noqa: Y015

@_typing.final
class RequestListDir(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PATH_FIELD_NUMBER: _builtins.int
    path: _builtins.str
    def __init__(
        self,
        *,
        path: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["path", b"path"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestListDir: _TypeAlias = RequestListDir  # noqa: Y015

@_typing.final
class RequestDummyBlock(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestDummyBlock: _TypeAlias = RequestDummyBlock  # noqa: Y015

@_typing.final
class ReplyDummyBlock(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    size: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        size: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDummyBlock: _TypeAlias = ReplyDummyBlock  # noqa: Y015

@_typing.final
class RequestGetClassList(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestGetClassList: _TypeAlias = RequestGetClassList  # noqa: Y015

@_typing.final
class ObjcClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    NAME_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    name: _builtins.str
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        name: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "name", b"name"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ObjcClass: _TypeAlias = ObjcClass  # noqa: Y015

@_typing.final
class ReplyGetClassList(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    CLASSES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def classes(self) -> _containers.RepeatedCompositeFieldContainer[Global___ObjcClass]: ...
    def __init__(
        self,
        *,
        classes: _abc.Iterable[Global___ObjcClass] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["classes", b"classes"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyGetClassList: _TypeAlias = ReplyGetClassList  # noqa: Y015

@_typing.final
class ReplyError(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: _builtins.int
    message: _builtins.str
    def __init__(
        self,
        *,
        message: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["message", b"message"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyError: _TypeAlias = ReplyError  # noqa: Y015

@_typing.final
class ReplyListDir(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DIR_ENTRIES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def dir_entries(self) -> _containers.RepeatedCompositeFieldContainer[Global___DirEntry]: ...
    def __init__(
        self,
        *,
        dir_entries: _abc.Iterable[Global___DirEntry] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["dir_entries", b"dir_entries"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyListDir: _TypeAlias = ReplyListDir  # noqa: Y015

@_typing.final
class RequestCloseClient(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestCloseClient: _TypeAlias = RequestCloseClient  # noqa: Y015

@_typing.final
class ReplyCloseClient(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___ReplyCloseClient: _TypeAlias = ReplyCloseClient  # noqa: Y015

@_typing.final
class DirEntryStat(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ERRNO1_FIELD_NUMBER: _builtins.int
    ST_DEV_FIELD_NUMBER: _builtins.int
    ST_MODE_FIELD_NUMBER: _builtins.int
    ST_NLINK_FIELD_NUMBER: _builtins.int
    ST_INO_FIELD_NUMBER: _builtins.int
    ST_UID_FIELD_NUMBER: _builtins.int
    ST_GID_FIELD_NUMBER: _builtins.int
    ST_RDEV_FIELD_NUMBER: _builtins.int
    ST_SIZE_FIELD_NUMBER: _builtins.int
    ST_BLOCKS_FIELD_NUMBER: _builtins.int
    ST_BLKSIZE_FIELD_NUMBER: _builtins.int
    ST_ATIME1_FIELD_NUMBER: _builtins.int
    ST_MTIME1_FIELD_NUMBER: _builtins.int
    ST_CTIME1_FIELD_NUMBER: _builtins.int
    errno1: _builtins.int
    st_dev: _builtins.int
    st_mode: _builtins.int
    st_nlink: _builtins.int
    st_ino: _builtins.int
    st_uid: _builtins.int
    st_gid: _builtins.int
    st_rdev: _builtins.int
    st_size: _builtins.int
    st_blocks: _builtins.int
    st_blksize: _builtins.int
    st_atime1: _builtins.int
    st_mtime1: _builtins.int
    st_ctime1: _builtins.int
    def __init__(
        self,
        *,
        errno1: _builtins.int = ...,
        st_dev: _builtins.int = ...,
        st_mode: _builtins.int = ...,
        st_nlink: _builtins.int = ...,
        st_ino: _builtins.int = ...,
        st_uid: _builtins.int = ...,
        st_gid: _builtins.int = ...,
        st_rdev: _builtins.int = ...,
        st_size: _builtins.int = ...,
        st_blocks: _builtins.int = ...,
        st_blksize: _builtins.int = ...,
        st_atime1: _builtins.int = ...,
        st_mtime1: _builtins.int = ...,
        st_ctime1: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["errno1", b"errno1", "st_atime1", b"st_atime1", "st_blksize", b"st_blksize", "st_blocks", b"st_blocks", "st_ctime1", b"st_ctime1", "st_dev", b"st_dev", "st_gid", b"st_gid", "st_ino", b"st_ino", "st_mode", b"st_mode", "st_mtime1", b"st_mtime1", "st_nlink", b"st_nlink", "st_rdev", b"st_rdev", "st_size", b"st_size", "st_uid", b"st_uid"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DirEntryStat: _TypeAlias = DirEntryStat  # noqa: Y015

@_typing.final
class DirEntry(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    D_TYPE_FIELD_NUMBER: _builtins.int
    D_NAME_FIELD_NUMBER: _builtins.int
    LSTAT_FIELD_NUMBER: _builtins.int
    STAT_FIELD_NUMBER: _builtins.int
    d_type: _builtins.int
    d_name: _builtins.str
    @_builtins.property
    def lstat(self) -> Global___DirEntryStat: ...
    @_builtins.property
    def stat(self) -> Global___DirEntryStat: ...
    def __init__(
        self,
        *,
        d_type: _builtins.int = ...,
        d_name: _builtins.str = ...,
        lstat: Global___DirEntryStat | None = ...,
        stat: Global___DirEntryStat | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["lstat", b"lstat", "stat", b"stat"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["d_name", b"d_name", "d_type", b"d_type", "lstat", b"lstat", "stat", b"stat"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DirEntry: _TypeAlias = DirEntry  # noqa: Y015

@_typing.final
class ReturnRegistersArm(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    X0_FIELD_NUMBER: _builtins.int
    X1_FIELD_NUMBER: _builtins.int
    X2_FIELD_NUMBER: _builtins.int
    X3_FIELD_NUMBER: _builtins.int
    X4_FIELD_NUMBER: _builtins.int
    X5_FIELD_NUMBER: _builtins.int
    X6_FIELD_NUMBER: _builtins.int
    X7_FIELD_NUMBER: _builtins.int
    D0_FIELD_NUMBER: _builtins.int
    D1_FIELD_NUMBER: _builtins.int
    D2_FIELD_NUMBER: _builtins.int
    D3_FIELD_NUMBER: _builtins.int
    D4_FIELD_NUMBER: _builtins.int
    D5_FIELD_NUMBER: _builtins.int
    D6_FIELD_NUMBER: _builtins.int
    D7_FIELD_NUMBER: _builtins.int
    x0: _builtins.int
    x1: _builtins.int
    x2: _builtins.int
    x3: _builtins.int
    x4: _builtins.int
    x5: _builtins.int
    x6: _builtins.int
    x7: _builtins.int
    d0: _builtins.float
    d1: _builtins.float
    d2: _builtins.float
    d3: _builtins.float
    d4: _builtins.float
    d5: _builtins.float
    d6: _builtins.float
    d7: _builtins.float
    def __init__(
        self,
        *,
        x0: _builtins.int = ...,
        x1: _builtins.int = ...,
        x2: _builtins.int = ...,
        x3: _builtins.int = ...,
        x4: _builtins.int = ...,
        x5: _builtins.int = ...,
        x6: _builtins.int = ...,
        x7: _builtins.int = ...,
        d0: _builtins.float = ...,
        d1: _builtins.float = ...,
        d2: _builtins.float = ...,
        d3: _builtins.float = ...,
        d4: _builtins.float = ...,
        d5: _builtins.float = ...,
        d6: _builtins.float = ...,
        d7: _builtins.float = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["d0", b"d0", "d1", b"d1", "d2", b"d2", "d3", b"d3", "d4", b"d4", "d5", b"d5", "d6", b"d6", "d7", b"d7", "x0", b"x0", "x1", b"x1", "x2", b"x2", "x3", b"x3", "x4", b"x4", "x5", b"x5", "x6", b"x6", "x7", b"x7"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReturnRegistersArm: _TypeAlias = ReturnRegistersArm  # noqa: Y015

@_typing.final
class Argument(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    V_INT_FIELD_NUMBER: _builtins.int
    V_DOUBLE_FIELD_NUMBER: _builtins.int
    V_STR_FIELD_NUMBER: _builtins.int
    V_BYTES_FIELD_NUMBER: _builtins.int
    v_int: _builtins.int
    v_double: _builtins.float
    v_str: _builtins.str
    v_bytes: _builtins.bytes
    def __init__(
        self,
        *,
        v_int: _builtins.int = ...,
        v_double: _builtins.float = ...,
        v_str: _builtins.str = ...,
        v_bytes: _builtins.bytes = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["v_int", "v_double", "v_str", "v_bytes"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___Argument: _TypeAlias = Argument  # noqa: Y015



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: rpc_api.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'rpc_api.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rrpc_api.proto\x12\x07rpc.api\"$\n\x11RequestShowObject\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\"&\n\x0fReplyShowObject\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\"#\n\x10RequestShowClass\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\"%\n\x0eReplyShowClass\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\"/\n\rRequestDlopen\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0c\n\x04mode\x18\x02 \x01(\x05\"\x1d\n\x0bReplyDlopen\x12\x0e\n\x06handle\x18\x01 \x01(\x04\" \n\x0eRequestDlclose\x12\x0e\n\x06handle\x18\x01 \x01(\x04\"\x1b\n\x0cReplyDlclose\x12\x0b\n\x03res\x18\x01 \x01(\r\"3\n\x0cRequestDlsym\x12\x0e\n\x06handle\x18\x01 \x01(\x04\x12\x13\n\x0bsymbol_name\x18\x02 \x01(\t\"\x19\n\nReplyDlsym\x12\x0b\n\x03ptr\x18\x01 \x01(\x04\"=\n\x0bRequestExec\x12\x12\n\nbackground\x18\x01 \x01(\x08\x12\x0c\n\x04\x61rgv\x18\x02 \x03(\t\x12\x0c\n\x04\x65nvp\x18\x03 \x03(\t\"$\n\tReplyExec\x12\x0b\n\x03pid\x18\x01 \x01(\r\x12\n\n\x02\x66\x64\x18\x02 \x01(\x05\"V\n\x0bRequestCall\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x15\n\rva_list_index\x18\x02 \x01(\x04\x12\x1f\n\x04\x61rgv\x18\x03 \x03(\x0b\x32\x11.rpc.api.Argument\"j\n\tReplyCall\x12\x34\n\rarm_registers\x18\x01 \x01(\x0b\x32\x1b.rpc.api.ReturnRegistersArmH\x00\x12\x16\n\x0creturn_value\x18\x02 \x01(\x04H\x00\x42\x0f\n\rreturn_values\",\n\x0bRequestPeek\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\x04\"\x19\n\tReplyPeek\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\",\n\x0bRequestPoke\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x0b\n\tReplyPoke\"\x1e\n\x0eRequestListDir\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x13\n\x11RequestDummyBlock\"0\n\x0fReplyDummyBlock\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\x04\"\x15\n\x13RequestGetClassList\"*\n\tObjcClass\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04name\x18\x02 \x01(\t\"8\n\x11ReplyGetClassList\x12#\n\x07\x63lasses\x18\x01 \x03(\x0b\x32\x12.rpc.api.ObjcClass\"\x1d\n\nReplyError\x12\x0f\n\x07message\x18\x01 \x01(\t\"6\n\x0cReplyListDir\x12&\n\x0b\x64ir_entries\x18\x03 \x03(\x0b\x32\x11.rpc.api.DirEntry\"\x14\n\x12RequestCloseClient\"\x12\n\x10ReplyCloseClient\"\x83\x02\n\x0c\x44irEntryStat\x12\x0e\n\x06\x65rrno1\x18\x01 \x01(\x04\x12\x0e\n\x06st_dev\x18\x02 \x01(\x04\x12\x0f\n\x07st_mode\x18\x03 \x01(\x04\x12\x10\n\x08st_nlink\x18\x04 \x01(\x04\x12\x0e\n\x06st_ino\x18\x05 \x01(\x04\x12\x0e\n\x06st_uid\x18\x06 \x01(\x04\x12\x0e\n\x06st_gid\x18\x07 \x01(\x04\x12\x0f\n\x07st_rdev\x18\x08 \x01(\x04\x12\x0f\n\x07st_size\x18\t \x01(\x04\x12\x11\n\tst_blocks\x18\n \x01(\x04\x12\x12\n\nst_blksize\x18\x0b \x01(\x04\x12\x11\n\tst_atime1\x18\x0c \x01(\x04\x12\x11\n\tst_mtime1\x18\r \x01(\x04\x12\x11\n\tst_ctime1\x18\x0e \x01(\x04\"u\n\x08\x44irEntry\x12\x0e\n\x06\x64_type\x18\x01 \x01(\x04\x12\x0e\n\x06\x64_name\x18\x02 \x01(\t\x12$\n\x05lstat\x18\x03 \x01(\x0b\x32\x15.rpc.api.DirEntryStat\x12#\n\x04stat\x18\x04 \x01(\x0b\x32\x15.rpc.api.DirEntryStat\"\xd4\x01\n\x12ReturnRegistersArm\x12\n\n\x02x0\x18\x01 \x01(\x04\x12\n\n\x02x1\x18\x02 \x01(\x04\x12\n\n\x02x2\x18\x03 \x01(\x04\x12\n\n\x02x3\x18\x04 \x01(\x04\x12\n\n\x02x4\x18\x05 \x01(\x04\x12\n\n\x02x5\x18\x06 \x01(\x04\x12\n\n\x02x6\x18\x07 \x01(\x04\x12\n\n\x02x7\x18\x08 \x01(\x04\x12\n\n\x02\x64\x30\x18\t \x01(\x01\x12\n\n\x02\x64\x31\x18\n \x01(\x01\x12\n\n\x02\x64\x32\x18\x0b \x01(\x01\x12\n\n\x02\x64\x33\x18\x0c \x01(\x01\x12\n\n\x02\x64\x34\x18\r \x01(\x01\x12\n\n\x02\x64\x35\x18\x0e \x01(\x01\x12\n\n\x02\x64\x36\x18\x0f \x01(\x01\x12\n\n\x02\x64\x37\x18\x10 \x01(\x01\"[\n\x08\x41rgument\x12\x0f\n\x05v_int\x18\x01 \x01(\x04H\x00\x12\x12\n\x08v_double\x18\x02 \x01(\x01H\x00\x12\x0f\n\x05v_str\x18\x03 \x01(\tH\x00\x12\x11\n\x07v_bytes\x18\x04 \x01(\x0cH\x00\x42\x06\n\x04type*\x8c\x02\n\x05MsgId\x12\x12\n\x0eMSG_ID_UNKNOWN\x10\x00\x12\x0e\n\nREQ_DLOPEN\x10\x01\x12\x0f\n\x0bREQ_DLCLOSE\x10\x02\x12\r\n\tREQ_DLSYM\x10\x03\x12\x0c\n\x08REQ_PEEK\x10\x04\x12\x0c\n\x08REQ_POKE\x10\x05\x12\x0c\n\x08REQ_EXEC\x10\x06\x12\x0c\n\x08REQ_CALL\x10\x07\x12\x14\n\x10REQ_CLOSE_CLIENT\x10\x08\x12\x10\n\x0cREQ_LIST_DIR\x10\t\x12\x13\n\x0fREQ_DUMMY_BLOCK\x10\n\x12\x16\n\x12REQ_GET_CLASS_LIST\x10\x0b\x12\x13\n\x0fREQ_SHOW_OBJECT\x10\x0c\x12\x12\n\x0eREQ_SHOW_CLASS\x10\r\"\t\x08\x80\x02\x10\xff\xff\xff\xff\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rpc_api_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MSGID']._serialized_start=1879
  _globals['_MSGID']._serialized_end=2147
  _globals['_REQUESTSHOWOBJECT']._serialized_start=26
  _globals['_REQUESTSHOWOBJECT']._serialized_end=62
  _globals['_REPLYSHOWOBJECT']._serialized_start=64
  _globals['_REPLYSHOWOBJECT']._serialized_end=102
  _globals['_REQUESTSHOWCLASS']._serialized_start=104
  _globals['_REQUESTSHOWCLASS']._serialized_end=139
  _globals['_REPLYSHOWCLASS']._serialized_start=141
  _globals['_REPLYSHOWCLASS']._serialized_end=178
  _globals['_REQUESTDLOPEN']._serialized_start=180
  _globals['_REQUESTDLOPEN']._serialized_end=227
  _globals['_REPLYDLOPEN']._serialized_start=229
  _globals['_REPLYDLOPEN']._serialized_end=258
  _globals['_REQUESTDLCLOSE']._serialized_start=260
  _globals['_REQUESTDLCLOSE']._serialized_end=292
  _globals['_REPLYDLCLOSE']._serialized_start=294
  _globals['_REPLYDLCLOSE']._serialized_end=321
  _globals['_REQUESTDLSYM']._serialized_start=323
  _globals['_REQUESTDLSYM']._serialized_end=374
  _globals['_REPLYDLSYM']._serialized_start=376
  _globals['_REPLYDLSYM']._serialized_end=401
  _globals['_REQUESTEXEC']._serialized_start=403
  _globals['_REQUESTEXEC']._serialized_end=464
  _globals['_REPLYEXEC']._serialized_start=466
  _globals['_REPLYEXEC']._serialized_end=502
  _globals['_REQUESTCALL']._serialized_start=504
  _globals['_REQUESTCALL']._serialized_end=590
  _globals['_REPLYCALL']._serialized_start=592
  _globals['_REPLYCALL']._serialized_end=698
  _globals['_REQUESTPEEK']._serialized_start=700
  _globals['_REQUESTPEEK']._serialized_end=744
  _globals['_REPLYPEEK']._serialized_start=746
  _globals['_REPLYPEEK']._serialized_end=771
  _globals['_REQUESTPOKE']._serialized_start=773
  _globals['_REQUESTPOKE']._serialized_end=817
  _globals['_REPLYPOKE']._serialized_start=819
  _globals['_REPLYPOKE']._serialized_end=830
  _globals['_REQUESTLISTDIR']._serialized_start=832
  _globals['_REQUESTLISTDIR']._serialized_end=862
  _globals['_REQUESTDUMMYBLOCK']._serialized_start=864
  _globals['_REQUESTDUMMYBLOCK']._serialized_end=883
  _globals['_REPLYDUMMYBLOCK']._serialized_start=885
  _globals['_REPLYDUMMYBLOCK']._serialized_end=933
  _globals['_REQUESTGETCLASSLIST']._serialized_start=935
  _globals['_REQUESTGETCLASSLIST']._serialized_end=956
  _globals['_OBJCCLASS']._serialized_start=958
  _globals['_OBJCCLASS']._serialized_end=1000
  _globals['_REPLYGETCLASSLIST']._serialized_start=1002
  _globals['_REPLYGETCLASSLIST']._serialized_end=1058
  _globals['_REPLYERROR']._serialized_start=1060
  _globals['_REPLYERROR']._serialized_end=1089
  _globals['_REPLYLISTDIR']._serialized_start=1091
  _globals['_REPLYLISTDIR']._serialized_end=1145
  _globals['_REQUESTCLOSECLIENT']._serialized_start=1147
  _globals['_REQUESTCLOSECLIENT']._serialized_end=1167
  _globals['_REPLYCLOSECLIENT']._serialized_start=1169
  _globals['_REPLYCLOSECLIENT']._serialized_end=1187
  _globals['_DIRENTRYSTAT']._serialized_start=1190
  _globals['_DIRENTRYSTAT']._serialized_end=1449
  _globals['_DIRENTRY']._serialized_start=1451
  _globals['_DIRENTRY']._serialized_end=1568
  _globals['_RETURNREGISTERSARM']._serialized_start=1571
  _globals['_RETURNREGISTERSARM']._serialized_end=1783
  _globals['_ARGUMENT']._serialized_start=1785
  _globals['_ARGUMENT']._serialized_end=1876
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: rpc.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'rpc.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\trpc.proto\x12\x03rpc\"O\n\nRpcMessage\x12\r\n\x05magic\x18\x01 \x01(\r\x12\x11\n\tclient_id\x18\x02 \x01(\r\x12\x0e\n\x06msg_id\x18\x03 \x01(\r\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\">\n\rRpcPtyMessage\x12\x10\n\x06\x62uffer\x18\x01 \x01(\x0cH\x00\x12\x13\n\texit_code\x18\x02 \x01(\rH\x00\x42\x06\n\x04type\"\x92\x01\n\tHandshake\x12\r\n\x05magic\x18\x01 \x01(\r\x12\x17\n\x04\x61rch\x18\x02 \x01(\x0e\x32\t.rpc.Arch\x12\x0f\n\x07sysname\x18\x03 \x01(\t\x12\x0f\n\x07machine\x18\x04 \x01(\t\x12\x10\n\x08platform\x18\x05 \x01(\t\x12\x16\n\x0eserver_version\x18\x07 \x01(\r\x12\x11\n\tclient_id\x18\x08 \x01(\r*\x91\x01\n\x11ProtocolConstants\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x15\n\x0eSERVER_VERSION\x10\x91\x90\xa2\x44\x12\x14\n\rMESSAGE_MAGIC\x10\xe9\x8a\x8d\t\x12\x0e\n\tREP_ERROR\x10\x80 \x12\x19\n\x13RPC_PTY_BUFFER_SIZE\x10\x80\x80\x04\x12\x17\n\x12RPC_MAX_REQ_MSG_ID\x10\x80\x02*(\n\x04\x41rch\x12\x10\n\x0c\x41RCH_UNKNOWN\x10\x00\x12\x0e\n\nARCH_ARM64\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rpc_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_PROTOCOLCONSTANTS']._serialized_start=313
  _globals['_PROTOCOLCONSTANTS']._serialized_end=458
  _globals['_ARCH']._serialized_start=460
  _globals['_ARCH']._serialized_end=500
  _globals['_RPCMESSAGE']._serialized_start=18
  _globals['_RPCMESSAGE']._serialized_end=97
  _globals['_RPCPTYMESSAGE']._serialized_start=99
  _globals['_RPCPTYMESSAGE']._serialized_end=161
  _globals['_HANDSHAKE']._serialized_start=164
  _globals['_HANDSHAKE']._serialized_end=310
# @@protoc_insertion_point(module_scope)



from functools import cached_property

from rpcclient.clients.linux.structs import utsname
from rpcclient.core.client import CoreClient


class LinuxClient(CoreClient):
    @cached_property
    def uname(self):
        with self.safe_calloc(utsname.sizeof()) as uname:
            assert self.symbols.uname(uname) == 0
            return utsname.parse_stream(uname)



from construct import Bytes, Computed, Int8ul, Int16ul, Int32ul, Int64ul, PaddedString, Padding, Struct

_UTSNAME_LENGTH = 65
_D_NAME_LENGTH = 256

utsname = Struct(
    "sysname" / PaddedString(_UTSNAME_LENGTH, "utf8"),
    "nodename" / PaddedString(_UTSNAME_LENGTH, "utf8"),
    "release" / PaddedString(_UTSNAME_LENGTH, "utf8"),
    "version" / PaddedString(_UTSNAME_LENGTH, "utf8"),
    "machine" / PaddedString(_UTSNAME_LENGTH, "utf8"),
    "domainname" / PaddedString(_UTSNAME_LENGTH, "utf8"),
)

dirent = Struct(
    "d_ino" / Int32ul,
    Padding(4),
    "d_off" / Int64ul,
    "d_reclen" / Int16ul,
    "d_type" / Int8ul,
    "_d_name_bytes" / Bytes(_D_NAME_LENGTH),
    "d_name" / Computed(lambda x: x._d_name_bytes.split(b"\x00", 1)[0].decode()),
)



import pytest

pytestmark = pytest.mark.darwin


def test_from_xpc_object(client):
    """
    :param rpcclient.darwin.client.DarwinClient client:
    """
    criteria = client.symbols.xpc_dictionary_create(0, 0, 0)
    client.symbols.xpc_dictionary_set_int64(criteria, "Delay", 5)
    client.symbols.xpc_dictionary_set_int64(criteria, "GracePeriod", 1)
    client.symbols.xpc_dictionary_set_string(criteria, "Priority", "Utility")
    assert client.xpc.decode_xpc_object_using_cf_serialization(criteria) == {
        "Delay": 5,
        "Priority": "Utility",
        "GracePeriod": 1,
    }



from typing import TYPE_CHECKING

from rpcclient.clients.darwin.subsystems.cfpreferences import CFPreferences
from rpcclient.clients.darwin.subsystems.scpreferences import SCPreferences

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class Preferences:
    """Preferences utils"""

    def __init__(self, client: "DarwinClient"):
        """
        :param rpcclient.darwin.client.DarwinClient client:
        """
        self.cf = CFPreferences(client)
        self.sc = SCPreferences(client)



from pathlib import Path
from typing import TYPE_CHECKING

from rpcclient.clients.darwin.subsystems.crash_reports import CrashReports

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class Reports:
    """equivalent to the data that can be found using the Console app inside the Reports section"""

    def __init__(self, client: "DarwinClient", crash_reports_dir):
        self._client = client
        self.crash_reports = CrashReports(client, crash_reports_dir)

    def get_logs(self, prefix="") -> list[Path]:
        result = []
        sub_paths = ["var/log", "Library/Logs"]
        for sub_path in sub_paths:
            for path in self._client.roots:
                path = Path(path) / sub_path
                if not self._client.fs.accessible(path):
                    continue

                for root, _dirs, files in self._client.fs.walk(path, onerror=lambda x: None):
                    for name in files:
                        if not self._client.fs.accessible(path):
                            continue

                        if name.endswith(".log") and name.startswith(prefix):
                            result.append(Path(root) / name)
        return result

    @property
    def system_log(self) -> str:
        with self._client.fs.open("/var/log/system.log", "r") as f:
            return f.read().decode()



from typing import TYPE_CHECKING

from rpcclient.core.structs.consts import SIGTERM
from rpcclient.exceptions import BadReturnValueError

if TYPE_CHECKING:
    from rpcclient.core.client import CoreClient


class Processes:
    def __init__(self, client: "CoreClient"):
        """
        process manager
        :param rpcclient.darwin_client.Client client: client
        """
        self._client = client

    def kill(self, pid: int, sig: int = SIGTERM):
        """Send a signal to a remote process."""
        if self._client.symbols.kill(pid, sig) != 0:
            raise BadReturnValueError(f"kill({pid}, {sig}) failed ({self._client.last_error})")

    def waitpid(self, pid: int, flags: int = 0):
        """Wait for a remote process to change state and return status."""
        with self._client.safe_malloc(8) as stat_loc:
            err = self._client.symbols.waitpid(pid, stat_loc, flags).c_int64
            if err == -1:
                raise BadReturnValueError(f"waitpid(): returned {err} ({self._client.last_error})")
            return stat_loc[0]



import importlib
import logging
from typing import Any, Optional

from rpcclient.protos.rpc_api_pb2 import ReplyError
from rpcclient.protos.rpc_pb2 import ProtocolConstants
from rpcclient.registry import Registry

logger = logging.getLogger(__name__)


class RpcMessageRegistry:
    """
    Handles the registration of RPC message mappings.

    This class extends the Registry to manage mappings between unique integer
    protocol message IDs and their corresponding request/response message
    types. It provides utilities for dynamically loading these message
    mappings from specified modules, ensuring a streamlined mechanism for
    message resolution. The typical usage involves initializing the class,
    potentially with module paths, where message definitions are declared.

    Methods:
        load_by_module: Dynamically loads message ID to class mappings from the provided module.

    """

    def __init__(self, init_data: Optional[dict] = None, modules: Optional[list[str]] = None):
        self._messages: Registry[int, type[Any]] = Registry()
        self._messages.register(ProtocolConstants.REP_ERROR, ReplyError)
        if init_data is None:
            init_data = {}
        if modules is None:
            modules = []
        self._messages.update(init_data)
        for module in modules:
            self.load_from_module(module)

    def load_from_module(self, module_path: str) -> None:
        mod = importlib.import_module(module_path)

        # Use the module descriptor for robust access to enum values
        enum_desc = mod.DESCRIPTOR.enum_types_by_name["MsgId"]

        def _to_camel(s: str) -> str:
            # "DLSYM" -> "Dlsym", "DUMMY_BLOCK" -> "DummyBlock"
            return "".join(p.capitalize() for p in s.lower().split("_"))

        pair_count = 0
        for enum_value in enum_desc.values:
            name = enum_value.name
            value = enum_value.number
            if not name.startswith("REQ_"):
                continue
            camel = _to_camel(name.split("_", 1)[1])
            req_class = getattr(mod, f"Request{camel}", None)
            rep_class = getattr(mod, f"Reply{camel}", None)
            if req_class is None or rep_class is None:
                logger.warning(f"Skipping {name}: missing Request{camel} or Reply{camel} in {module_path}")
            pair_count += 1
            self.register_pair(value, req_class, rep_class)
        logger.debug(f"Loaded {pair_count} bindings from {module_path}")

    def register_pair(self, msg_id: int, req_class: type[Any], rep_class: type[Any]):
        if msg_id > ProtocolConstants.RPC_MAX_REQ_MSG_ID:
            raise ValueError(f"Invalid msg_id: {msg_id}")
        self._messages.register(msg_id, req_class)
        self._messages.register(msg_id + ProtocolConstants.RPC_MAX_REQ_MSG_ID, rep_class)

    def unregister_pair(self, msg_id: int):
        if msg_id > ProtocolConstants.RPC_MAX_REQ_MSG_ID:
            raise ValueError(f"Invalid msg_id: {msg_id}")
        self._messages.unregister(msg_id)
        self._messages.unregister(msg_id + ProtocolConstants.RPC_MAX_REQ_MSG_ID)

    def update(self, data: dict, overwrite: bool = False):
        self._messages.update(data, overwrite)

    def clone(self) -> "RpcMessageRegistry":
        new_registry = RpcMessageRegistry()
        new_registry.update(dict(self._messages.items()), overwrite=True)
        return new_registry

    def get(self, msg_id: int) -> type[Any]:
        msg_class = self._messages.get(msg_id)
        if msg_class is None:
            raise ValueError(f"Unknown msg_id: {msg_id}")
        return msg_class

    def clear(self) -> None:
        self._messages.clear()



from enum import Enum
from typing import TYPE_CHECKING, Optional

from rpcclient.exceptions import RpcPermissionError

if TYPE_CHECKING:
    from rpcclient.clients.darwin.client import DarwinClient


class CLAuthorizationStatus(Enum):
    kCLAuthorizationStatusNotDetermined = 0
    kCLAuthorizationStatusRestricted = 1
    kCLAuthorizationStatusDenied = 2
    kCLAuthorizationStatusAuthorizedAlways = 3
    kCLAuthorizationStatusAuthorizedWhenInUse = 4
    kCLAuthorizationStatusAuthorized = kCLAuthorizationStatusAuthorizedAlways

    @classmethod
    def from_value(cls, value: int):
        for i in cls:
            if i.value == value:
                return i


class Location:
    """
    Wrapper to CLLocationManager
    https://developer.apple.com/documentation/corelocation/cllocationmanager?language=objc
    """

    def __init__(self, client: "DarwinClient"):
        self._client = client
        self._client.load_framework("CoreLocation")

        self._CLLocationManager = self._client.symbols.objc_getClass("CLLocationManager")
        self._location_manager = self._CLLocationManager.objc_call("sharedManager")

    @property
    def location_services_enabled(self) -> bool:
        """opt-in status for location services"""
        return bool(self._location_manager.objc_call("locationServicesEnabled"))

    @location_services_enabled.setter
    def location_services_enabled(self, value: bool):
        """opt-in status for location services"""
        self._CLLocationManager.objc_call("setLocationServicesEnabled:", value)

    @property
    def authorization_status(self) -> CLAuthorizationStatus:
        """authorization status for current server process of accessing location services"""
        return CLAuthorizationStatus.from_value(self._location_manager.objc_call("authorizationStatus"))

    @property
    def last_sample(self) -> Optional[dict]:
        """last taken location sample (or None if there isn't any)"""
        location = self._location_manager.objc_call("location")
        if not location:
            return None
        return location.objc_call("jsonObject").py()

    def request_always_authorization(self) -> None:
        """Request authorization to always query location data"""
        self._location_manager.objc_call("requestAlwaysAuthorization")

    def start_updating_location(self):
        """request location updates from CLLocationManager"""
        if self.authorization_status.value < CLAuthorizationStatus.kCLAuthorizationStatusAuthorizedAlways.value:
            raise RpcPermissionError()
        self._location_manager.objc_call("startUpdatingLocation")

    def stop_updating_location(self):
        """stop requesting location updates from CLLocationManager"""
        self._location_manager.objc_call("stopUpdatingLocation")

    def request_oneshot_location(self):
        """requests the one-time delivery of the user's current location"""
        if self.authorization_status.value < CLAuthorizationStatus.kCLAuthorizationStatusAuthorizedAlways.value:
            raise RpcPermissionError()
        self._location_manager.objc_call("requestLocation")
